import java.io.*;
import java.text.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;
import java.awt.image.*;
import java.awt.datatransfer.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.text.rtf.*;
import javax.swing.undo.*;
import javax.swing.plaf.*;
import javax.swing.filechooser.FileFilter;

public class TwoContentArea extends JTextPane implements Printable
{
  protected JFrame parentWindow;
  protected File currentDirectory = new File(System.getProperty("user.dir"));
  protected File currentFile;
  protected File selectedFile = null;
  protected TwoStyledDocument currentDocument;

  // Menu Items (to allow enable/disable of menu items)
  public JMenuItem undoItem, undoPopupItem, redoItem, redoPopupItem;
  public JMenuItem cutItem, cutPopupItem, copyItem, copyPopupItem;
  public JMenuItem pasteItem, pastePopupItem, deleteItem, deletePopupItem, findNextItem;
  public JRadioButtonMenuItem justifyLeft, justifyCenter, justifyRight;
  public JRadioButtonMenuItem popupJustifyLeft, popupJustifyCenter, popupJustifyRight;

  public TwoContentArea(JFrame myParent, File initialFile)
  {
    super();
    parentWindow = myParent;
    systemClipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
    setIgnoreChanges(-8);
    if(initialFile == null)
      newDocument();
    else
      openDocument(initialFile);
  }

  protected JPopupMenu popupMenu;
  public void setPopupMenu(JPopupMenu inputMenu)
  {
    popupMenu = inputMenu;
    add(popupMenu);
  }
  public void processMouseEvent(MouseEvent event)
  {
    if(event.isPopupTrigger()) { popupMenu.show(event.getComponent(), event.getX(), event.getY()); }
    else { super.processMouseEvent(event); }
  }

  public void newDocument()
  {
    Font currentFont = getFont();
    currentFile = null;
    currentDocument = new TwoStyledDocument();
    setDocument(currentDocument);
    try { currentDocument.insertString(0, " ", null); } catch(BadLocationException e) { }
    setFont(null);
    setFont(currentFont);
    try { currentDocument.remove(0, 1); } catch(BadLocationException e) { }
    currentDocument.addUndoableEditListener(new UndoHandler());
    undoManager = new UndoManager();
    currentDocument.addDocumentListener(new DocumentListener()
    {
      public void changedUpdate(DocumentEvent e) { setDirty(); }
      public void insertUpdate(DocumentEvent e) { setDirty(); }
      public void removeUpdate(DocumentEvent e) { setDirty(); }
    });
    setClean();
    setIgnoreChanges(8);
  }

  public void openDocument(File openFile)
  {
    Font currentFont = getFont();
    if(openFile == null)
    {
      JFileChooser openDialog = new JFileChooser(currentDirectory);
      FileFilter defaultFilter = new FileFilter()
      {
        public String getDescription() { return "2Pad File (*.two)"; }
        public boolean accept(File file)
        {
          if(file != null)
          {
            String fileName = file.getName().toLowerCase();
            if(file.isDirectory() || fileName.endsWith(".two"))
              return true;
          }
          return false;
        }
      };
      openDialog.addChoosableFileFilter(defaultFilter);
      openDialog.addChoosableFileFilter(new FileFilter()
      {
        public String getDescription() { return "Plain Text File (*.txt)"; }
        public boolean accept(File file)
        {
          if(file != null)
          {
            String fileName = file.getName().toLowerCase();
            if(file.isDirectory() || fileName.endsWith(".txt"))
              return true;
          }
          return false;
        }
      });
      openDialog.addChoosableFileFilter(new FileFilter()
      {
        public String getDescription() { return "Rich Text File (*.rtf)"; }
        public boolean accept(File file)
        {
          if(file != null)
          {
            String fileName = file.getName().toLowerCase();
            if(file.isDirectory() || fileName.endsWith(".rtf"))
              return true;
          }
          return false;
        }
      });
      openDialog.setFileFilter(defaultFilter);
      if(JFileChooser.APPROVE_OPTION != openDialog.showOpenDialog(parentWindow))
      {
        currentDirectory = openDialog.getCurrentDirectory();
        return;
      }
      currentDirectory = openDialog.getCurrentDirectory();
      openFile = openDialog.getSelectedFile();
      if(!openFile.getName().toLowerCase().endsWith(".two") && !openFile.getName().toLowerCase().endsWith(".txt") && !openFile.getName().toLowerCase().endsWith(".rtf"))
      {
        if(openDialog.getFileFilter().getDescription().equals("2Pad File (*.two)"))
          openFile = new File(openFile.getAbsolutePath().concat(".two"));
        if(openDialog.getFileFilter().getDescription().equals("Plain Text File (*.txt)"))
          openFile = new File(openFile.getAbsolutePath().concat(".txt"));
        if(openDialog.getFileFilter().getDescription().equals("Rich Text File (*.rtf)"))
          openFile = new File(openFile.getAbsolutePath().concat(".rtf"));
      }
    }
    try
    {
      if(openFile.getName().toLowerCase().endsWith(".two"))
      {
        FileInputStream inputFile = new FileInputStream(openFile);
        ObjectInputStream inputStream = new ObjectInputStream(inputFile);
        currentDocument = (TwoStyledDocument) inputStream.readObject();
        inputStream.close();
        inputFile.close();
      }
      else if(openFile.getName().toLowerCase().endsWith(".txt"))
      {
        currentDocument = new TwoStyledDocument();
        FileInputStream inputFile = new FileInputStream(openFile);
        (new DefaultEditorKit()).read(inputFile, currentDocument, 0);
        inputFile.close();
      }
      else if(openFile.getName().toLowerCase().endsWith(".rtf"))
      {
        currentDocument = new TwoStyledDocument();
        FileInputStream inputFile = new FileInputStream(openFile);
        (new RTFEditorKit()).read(inputFile, currentDocument, 0);
        inputFile.close();
      }
      else
      {
        try
        {
          FileInputStream inputFile = new FileInputStream(openFile);
          ObjectInputStream inputStream = new ObjectInputStream(inputFile);
          currentDocument = (TwoStyledDocument) inputStream.readObject();
          inputStream.close();
          inputFile.close();
        }
        catch(Exception e)
        {
          currentDocument = new TwoStyledDocument();
          FileInputStream inputFile = new FileInputStream(openFile);
          (new DefaultEditorKit()).read(inputFile, currentDocument, 0);
          inputFile.close();
        }
      }
    }
    catch(Exception e)
    {
      String message = new String("Unable to open file \"" + openFile.getName() + "\"\n" + e);
      JOptionPane.showMessageDialog(parentWindow, message, "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    currentFile = openFile;
    setDocument(currentDocument);
    try { currentDocument.insertString(0, " ", null); } catch(BadLocationException e) { }
    setFont(null);
    setFont(currentFont);
    try { currentDocument.remove(0, 1); } catch(BadLocationException e) { }
    currentDocument.addUndoableEditListener(new UndoHandler());
    undoManager = new UndoManager();
    currentDocument.addDocumentListener(new DocumentListener()
    {
      public void changedUpdate(DocumentEvent e) { setDirty(); }
      public void insertUpdate(DocumentEvent e) { setDirty(); }
      public void removeUpdate(DocumentEvent e) { setDirty(); }
    });
    setClean();
    setIgnoreChanges(8);
  }

  public boolean saveDocument()
  {
    selectedFile = currentFile;
    return saveDocumentAs();
  }

  public boolean saveDocumentAs()
  {
    if(selectedFile == null)
    {
      JFileChooser saveAsDialog = new JFileChooser(currentDirectory);
      FileFilter defaultFilter = new FileFilter()
      {
        public String getDescription() { return "2Pad File (*.two)"; }
        public boolean accept(File file)
        {
          if(file != null)
          {
            String fileName = file.getName().toLowerCase();
            if(file.isDirectory() || fileName.endsWith(".two"))
              return true;
          }
          return false;
        }
      };
      saveAsDialog.addChoosableFileFilter(defaultFilter);
      saveAsDialog.addChoosableFileFilter(new FileFilter()
      {
        public String getDescription() { return "Plain Text File (*.txt)"; }
        public boolean accept(File file)
        {
          if(file != null)
          {
            String fileName = file.getName().toLowerCase();
            if(file.isDirectory() || fileName.endsWith(".txt"))
              return true;
          }
          return false;
        }
      });
      saveAsDialog.addChoosableFileFilter(new FileFilter()
      {
        public String getDescription() { return "Rich Text File (*.rtf)"; }
        public boolean accept(File file)
        {
          if(file != null)
          {
            String fileName = file.getName().toLowerCase();
            if(file.isDirectory() || fileName.endsWith(".rtf"))
              return true;
          }
          return false;
        }
      });
      saveAsDialog.setFileFilter(defaultFilter);
      if(JFileChooser.APPROVE_OPTION != saveAsDialog.showSaveDialog(parentWindow))
      {
        currentDirectory = saveAsDialog.getCurrentDirectory();
        return false;
      }
      currentDirectory = saveAsDialog.getCurrentDirectory();
      selectedFile = saveAsDialog.getSelectedFile();
      if(!selectedFile.getName().toLowerCase().endsWith(".two") && !selectedFile.getName().toLowerCase().endsWith(".txt") && !selectedFile.getName().toLowerCase().endsWith(".rtf"))
      {
        if(saveAsDialog.getFileFilter().getDescription().equals("2Pad File (*.two)"))
          selectedFile = new File(selectedFile.getAbsolutePath().concat(".two"));
        if(saveAsDialog.getFileFilter().getDescription().equals("Plain Text File (*.txt)"))
          selectedFile = new File(selectedFile.getAbsolutePath().concat(".txt"));
        if(saveAsDialog.getFileFilter().getDescription().equals("Rich Text File (*.rtf)"))
          selectedFile = new File(selectedFile.getAbsolutePath().concat(".rtf"));
      }
      if(selectedFile.exists())
      {
        String message = new String("\"" + selectedFile.getName() + "\" already exists.\n\nOverwrite?");
        int selection = JOptionPane.showConfirmDialog(parentWindow, message, "Overwrite?", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);
        if(selection != JOptionPane.YES_OPTION)
        {
          selectedFile = null;
          return false;
        }
      }
    }
    try
    {
      if(selectedFile.getName().toLowerCase().endsWith(".txt"))
      {
        FileOutputStream outputFile = new FileOutputStream(selectedFile);
        (new DefaultEditorKit()).write(outputFile, currentDocument, 0, currentDocument.getLength());
        outputFile.close();
      }
      else if(selectedFile.getName().toLowerCase().endsWith(".rtf"))
      {
        FileOutputStream outputFile = new FileOutputStream(selectedFile);
        (new RTFEditorKit()).write(outputFile, currentDocument, 0, currentDocument.getLength());
        outputFile.close();
      }
      else
      {
        FileOutputStream outputFile = new FileOutputStream(selectedFile);
        ObjectOutputStream outputStream = new ObjectOutputStream(outputFile);
        outputStream.writeObject(currentDocument);
        outputStream.flush();
        outputStream.close();
        outputFile.close();
      }
    }
    catch(Exception e)
    {
      String message = new String("Unable to save to file \"" + selectedFile.getName() + "\"\n" + e);
      JOptionPane.showMessageDialog(parentWindow, message, "Error", JOptionPane.ERROR_MESSAGE);
      selectedFile = null;
      return false;
    }
    currentFile = selectedFile;
    selectedFile = null;
    setClean();
    return true;
  }

  public PageFormat currentFormat = new PageFormat();
  protected int currentPrintJobPages;
  protected PrintView currentPrintJob = null;
  class PrintView extends BoxView
  {
    public PrintView(Element element, View parent, double width, double height)
    {
      super(element, Y_AXIS);
      setParent(parent);
      setSize((int)width, (int)height);
      layout((int)width, (int)height);
      currentPrintJobPages = (int)java.lang.Math.ceil((double)(getOffset(Y_AXIS, getViewCount() - 1) + 1 + getSpan(Y_AXIS, getViewCount() - 1)) / height);
    }
  }
  public int print(Graphics printImage, PageFormat pageFormat, int pageIndex)
  {
    if(currentPrintJob == null)
      { currentPrintJob = new PrintView(currentDocument.getDefaultRootElement(), getUI().getRootView(this), pageFormat.getImageableWidth(), pageFormat.getImageableHeight()); }
    if(pageIndex >= currentPrintJobPages)
    {
      currentPrintJob = null;
      return Printable.NO_SUCH_PAGE;
    }
    printImage.translate((int)pageFormat.getImageableX(), (int)pageFormat.getImageableY());
    printImage.setClip(0, 0, (int)pageFormat.getImageableWidth(), (int)pageFormat.getImageableHeight());
    currentPrintJob.paint(printImage, new Rectangle(0, -(pageIndex * (int)pageFormat.getImageableHeight()), (int)pageFormat.getImageableWidth(), (int)pageFormat.getImageableHeight()));
    return Printable.PAGE_EXISTS;
  }

  public boolean closeDocument(boolean showCancelButton)
  {
    if(isDirty())
    {
      String message = new String("\"" + getCurrentFileName() + "\" has been modified.\n\nDo you want to save the changes?");
      int selection = JOptionPane.showConfirmDialog(parentWindow, message, "Save Changes?", (showCancelButton ? JOptionPane.YES_NO_CANCEL_OPTION : JOptionPane.YES_NO_OPTION), JOptionPane.WARNING_MESSAGE);
      if(selection == JOptionPane.YES_OPTION)
        return saveDocument();
      if(selection == JOptionPane.NO_OPTION)
        return true;
      return false;
    }
    return true;
  }

  public UndoManager undoManager;
  class UndoHandler implements UndoableEditListener { public void undoableEditHappened(UndoableEditEvent event) { undoManager.addEdit(event.getEdit()); } }
  protected UndoActionListener undo = new UndoActionListener();
  class UndoActionListener implements ActionListener { public void actionPerformed(ActionEvent event) { try
  {
    String lastUndoName = undoManager.getUndoPresentationName();
    while(lastUndoName.equals(undoManager.getUndoPresentationName())) { undoManager.undo(); }
  } catch(CannotUndoException e) { } } }
  protected RedoActionListener redo = new RedoActionListener();
  class RedoActionListener implements ActionListener { public void actionPerformed(ActionEvent event) { try
  {
    String lastRedoName = undoManager.getRedoPresentationName();
    while(lastRedoName.equals(undoManager.getRedoPresentationName())) { undoManager.redo(); }
  } catch(CannotRedoException e) { } } }

  protected Clipboard systemClipboard;
  class ImageView extends BoxView
  {
    public int imageHeight;
    public ImageView(Element element, View parent, double width)
    {
      super(element, Y_AXIS);
      setParent(parent);
      setSize((int)width, 10);
      layout((int)width, 10);
      imageHeight = getOffset(Y_AXIS, getViewCount() - 1) + 1 + getSpan(Y_AXIS, getViewCount() - 1);
    }
  }
  class NotepadClipboardObject extends Object implements TerpClipboardInterface, Transferable, ClipboardOwner
  {
    protected String text;
    protected AttributeSet[] characterAttributes;
    protected AttributeSet[] paragraphAttributes;
    protected int imageWidth;
    protected View rootView;
    protected DataFlavor[] supportedFlavors = {DataFlavor.stringFlavor};
    public NotepadClipboardObject(String myText, AttributeSet[] myCharacterAttributes, AttributeSet[] myParagraphAttributes, int myImageWidth, View myRootView)
    {
      text = myText;
      characterAttributes = myCharacterAttributes;
      paragraphAttributes = myParagraphAttributes;
      imageWidth = myImageWidth;
      rootView = myRootView;
    }
    public int copiedFrom() { return TerpClipboardInterface.NOTEPAD; }
    public String getString() { return text; }
    public AttributeSet[] getCharacterAttributes() { return characterAttributes; }
    public AttributeSet[] getParagraphAttributes() { return paragraphAttributes; }
    public BufferedImage getImage()
    {
      TwoStyledDocument imageDocument = new TwoStyledDocument();
//      try { imageDocument.insertString(0, getString(), null); } catch(BadLocationException e) { }
      imageDocument.twoPasteStyledText(0, getString(), getCharacterAttributes(), getParagraphAttributes());
      ImageView imageView = new ImageView(imageDocument.getDefaultRootElement(), rootView, imageWidth);
      BufferedImage imageData = new BufferedImage(imageWidth, imageView.imageHeight, BufferedImage.TYPE_INT_RGB);
      Graphics imageGraphics = imageData.getGraphics();
      imageGraphics.setClip(0, 0, imageWidth, imageView.imageHeight);
      imageGraphics.setColor(Color.white);
      imageGraphics.fillRect(0, 0, imageWidth, imageView.imageHeight);
      imageView.paint(imageGraphics, new Rectangle(0, 0, imageWidth, imageView.imageHeight));
      return imageData;
    }
    public boolean isDataFlavorSupported(DataFlavor flavor) { return flavor.equals(DataFlavor.stringFlavor); }
    public DataFlavor[] getTransferDataFlavors() { return supportedFlavors; }
    public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException
    {
      if(flavor.equals(DataFlavor.stringFlavor)) return getString();
      else throw(new UnsupportedFlavorException(flavor));
    }
    public void lostOwnership(Clipboard clipboard, Transferable contents) { }
  }
  public void cut()
  {
    int selectionStart = getSelectionStart();
    int selectionEnd = getSelectionEnd();
    copy();
    try { currentDocument.remove(selectionStart, selectionEnd - selectionStart); } catch(BadLocationException e) { }
  }
  public void copy()
  {
    String copyText = getSelectedText();
    if(copyText == null) return;
    // I create this array to have one entry per copied character - this is something of a waste of space, but
    // it is necessary since there is no easy way to tell exactly how many attributes will need to be stored
    // and there is no easy way to tell where attributes should be restored when everything is pasted
    AttributeSet[] copyCharacterAttributes = new AttributeSet[getSelectionEnd() - getSelectionStart()];
    for(int counter = getSelectionStart(); counter < getSelectionEnd(); counter++)
    {
      copyCharacterAttributes[counter - getSelectionStart()] = currentDocument.getCharacterElement(counter).getAttributes().copyAttributes();
      counter = currentDocument.getCharacterElement(counter).getEndOffset() - 1;
    }
    int startIndex = currentDocument.getDefaultRootElement().getElementIndex(getSelectionStart());
    int endIndex = currentDocument.getDefaultRootElement().getElementIndex(getSelectionEnd() - 1);
    AttributeSet[] copyParagraphAttributes = new AttributeSet[endIndex - startIndex + 1];
    for(int counter = startIndex; counter <= endIndex; counter++)
      { copyParagraphAttributes[counter - startIndex] = currentDocument.getDefaultRootElement().getElement(counter).getAttributes().copyAttributes(); }
    NotepadClipboardObject copyObject = new NotepadClipboardObject(copyText, copyCharacterAttributes, copyParagraphAttributes, getWidth(), getUI().getRootView(this));
    systemClipboard.setContents(copyObject, copyObject);
  }
  public void paste()
  {
    Transferable pasteTransferable = systemClipboard.getContents(this);
    if(pasteTransferable instanceof NotepadClipboardObject)
    {
      NotepadClipboardObject pasteObject = (NotepadClipboardObject)pasteTransferable;
      replaceSelection("");
      currentDocument.twoPasteStyledText(getCaretPosition(), pasteObject.getString(), pasteObject.getCharacterAttributes(), pasteObject.getParagraphAttributes());
      return;
    }
    if(pasteTransferable instanceof TerpClipboardInterface)
    {
      TerpClipboardInterface pasteObject = (TerpClipboardInterface)pasteTransferable;
      if(pasteObject.copiedFrom() == TerpClipboardInterface.CALC || pasteObject.copiedFrom() == TerpClipboardInterface.INTEGRATOR)
        replaceSelection(pasteObject.getString());
      if(pasteObject.copiedFrom() == TerpClipboardInterface.SPREADSHEET || pasteObject.copiedFrom() == TerpClipboardInterface.OOD || pasteObject.copiedFrom() == TerpClipboardInterface.PAINT)
      {
        replaceSelection("\n");
        insertIcon(new ImageIcon(pasteObject.getImage()));
        replaceSelection("\n");
      }
      return;
    }
  }

  protected String staticFindString;
  protected boolean staticMatchCase;
  protected boolean staticDownDirection;
  public boolean find(String findString, boolean matchCase, boolean downDirection)
  {
    staticFindString = findString;
    staticMatchCase = matchCase;
    staticDownDirection = downDirection;
    findNextItem.setEnabled(true);

    String checkText;
    int position = getCaretPosition();
    int direction = 1;
    if(!downDirection)
    {
      position = getSelectionStart() - findString.length();
      direction = -1;
    }
    for(int counter = position; counter < currentDocument.getLength() + 1; counter += direction)
    {
      try { checkText = currentDocument.getText(counter, findString.length()); }
      catch(BadLocationException e) { return false; }
      if((!matchCase && checkText.equalsIgnoreCase(findString)) || (matchCase && checkText.equals(findString)))
      {
        setCaretPosition(counter);
        moveCaretPosition(counter + findString.length());
        return true;
      }
    }
    return false;
  }
  public boolean replace(String findString, String replaceString, boolean matchCase)
  {
    if(getSelectedText() != null && ((!matchCase && getSelectedText().equalsIgnoreCase(findString)) || (matchCase && getSelectedText().equals(findString))))
      replaceSelection(replaceString);
    return find(findString, matchCase, true);
  }
  public void replaceAll(String findString, String replaceString, boolean matchCase)
  {
    setCaretPosition(0);
    if(findString.equals("")) { return; }
    while(find(findString, matchCase, true)) { replaceSelection(replaceString); }
    setCaretPosition(0);
  }
  public void goToLine(int lineNumber) throws BadLocationException
  {
    if(lineNumber == 1)
    {
      setCaretPosition(0);
      return;
    }

    int currentLine = 1;
    for(int counter = 0; counter < currentDocument.getLength() + 2; counter++)
    {
      if(currentDocument.getText(counter, 1).equals("\n"))
      {
        currentLine++;
        if(currentLine == lineNumber)
        {
          setCaretPosition(counter + 1);
          break;
        }
      }
    }
  }

  public void insertTimeDate()
  {
    Calendar rightNow = Calendar.getInstance();
    DateFormat dte = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT);
    replaceSelection(dte.format(rightNow.getTime()));
  }

  public void insertImage()
  {
    JFileChooser insertDialog = new JFileChooser(currentDirectory);
    insertDialog.setFileFilter(new FileFilter()
    {
      public String getDescription() { return "Image File (*.jpg, *.jpeg, *.gif)"; }
      public boolean accept(File file)
      {
        if(file != null)
        {
          String fileName = file.getName().toLowerCase();
          if(file.isDirectory() || fileName.endsWith(".jpg") || fileName.endsWith(".jpeg") || fileName.endsWith(".gif"))
            return true;
        }
        return false;
      }
    });
    if(JFileChooser.APPROVE_OPTION == insertDialog.showOpenDialog(parentWindow))
    {
      File imageFile = insertDialog.getSelectedFile();
      try
      {
        replaceSelection("\n");
        insertIcon(new ImageIcon(imageFile.getAbsolutePath()));
        replaceSelection("\n");
      }
      catch(Exception e)
      {
        String message = new String("Unable to insert image \"" + imageFile.getName() + "\"\n" + e);
        JOptionPane.showMessageDialog(parentWindow, message, "Error", JOptionPane.ERROR_MESSAGE);
      }
    }
    currentDirectory = insertDialog.getCurrentDirectory();
  }

  protected boolean wordWrapEnabled;
  public void setWordWrapState(boolean state)
  {
    wordWrapEnabled = state;
    revalidate();
  }
  public boolean isWordWrapEnabled() { return wordWrapEnabled; }
  public boolean getScrollableTracksViewportWidth()
  {
    if(!wordWrapEnabled) return false;
    return super.getScrollableTracksViewportWidth();
  }
  JScrollPane scrollPane;
  public Dimension getPreferredSize()
  {
    Dimension newDimensions = super.getPreferredSize();
    newDimensions.width = Math.max(newDimensions.width, scrollPane.getViewportBorderBounds().width);
    return newDimensions;
  }

  protected boolean dirtyFlag;
  protected int ignoreChanges = 0;
  protected boolean isDirty() { return dirtyFlag; }
  protected boolean isClean() { return !dirtyFlag; }
  protected void setDirty()
  {
    if(ignoreChanges != 0) { ignoreChanges--; }
    else if(!dirtyFlag)
    {
      dirtyFlag = true;
      updateTitleBar();
    }
  }
  protected void setIgnoreChanges(int ignoreNumber) { ignoreChanges += ignoreNumber; }
  protected void setClean()
  {
    dirtyFlag = false;
    updateTitleBar();
  }

  protected String getCurrentFileName()
  {
    if(currentFile == null)
      return new String("Untitled");
    else
      return new String(currentFile.getName());
  }

  protected void updateTitleBar()
  {
    if(isClean())
      parentWindow.setTitle("2Pad - " + getCurrentFileName());
    else
      parentWindow.setTitle("2Pad - " + getCurrentFileName() + " *");
  }
}
