import java.io.*;
import java.awt.event.*;
import java.awt.print.*;
import java.awt.datatransfer.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.border.*;
import javax.swing.event.*;

public class TwoMenus
{
  public static String version = new String("Version 1.1.2");
  protected TwoPad parentWindow;
  protected TwoContentArea contentArea;

  public TwoMenus(TwoPad myParent, TwoContentArea myContentArea)
  {
    contentArea = myContentArea;
    parentWindow = myParent;

    createMenuBar();
    createPopupMenu();
  }

  protected void createMenuBar()
  {
    JMenuBar menuBar = new JMenuBar();

    // File Menu
    JMenu fileMenu = new JMenu("File");
    fileMenu.setMnemonic('F');
      // New
      JMenuItem newItem = new JMenuItem("New");
      newItem.setMnemonic('N');
      newItem.setAccelerator(KeyStroke.getKeyStroke("control N"));
      newItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
      {
        if(contentArea.closeDocument(true))
          contentArea.newDocument();
      } });
      fileMenu.add(newItem);
      // Open
      JMenuItem openItem = new JMenuItem("Open...");
      openItem.setMnemonic('O');
      openItem.setAccelerator(KeyStroke.getKeyStroke("control O"));
      openItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
      {
        if(contentArea.closeDocument(true))
          contentArea.openDocument(null);
      } });
      fileMenu.add(openItem);
      // Save
      JMenuItem saveItem = new JMenuItem("Save");
      saveItem.setMnemonic('S');
      saveItem.setAccelerator(KeyStroke.getKeyStroke("control S"));
      saveItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { contentArea.saveDocument(); } });
      fileMenu.add(saveItem);
      // Save As
      JMenuItem saveAsItem = new JMenuItem("Save As...");
      saveAsItem.setMnemonic('A');
      saveAsItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { contentArea.saveDocumentAs(); } });
      fileMenu.add(saveAsItem);
      // Separtator
      fileMenu.addSeparator();
      // Page Setup
      JMenuItem pageSetupItem = new JMenuItem("Page Setup...");
      pageSetupItem.setMnemonic('U');
      pageSetupItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { contentArea.currentFormat = PrinterJob.getPrinterJob().pageDialog(contentArea.currentFormat); } });
      fileMenu.add(pageSetupItem);
      // Print
      JMenuItem printItem = new JMenuItem("Print...");
      printItem.setMnemonic('P');
      printItem.setAccelerator(KeyStroke.getKeyStroke("control P"));
      printItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
      {
        try
        {
          PrinterJob printJob = PrinterJob.getPrinterJob();
          printJob.setPrintable(contentArea, contentArea.currentFormat);
          if(!printJob.printDialog())
            return;
          printJob.print();
        }
        catch(Exception e)
        {
          String message = new String("Error while printing document:\n" + e);
          JOptionPane.showMessageDialog(parentWindow, message, "Error", JOptionPane.ERROR_MESSAGE);
        }
      } });
      fileMenu.add(printItem);
      // Separator
      fileMenu.addSeparator();
      // Exit
      JMenuItem exitItem = new JMenuItem("Exit");
      exitItem.setMnemonic('X');
      exitItem.setAccelerator(KeyStroke.getKeyStroke("alt F4"));
      exitItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { parentWindow.close(); } });
      fileMenu.add(exitItem);
    menuBar.add(fileMenu);
    // Edit Menu
    JMenu editMenu = new JMenu("Edit");
    editMenu.setMnemonic('E');
      // Undo
      JMenuItem undoItem = new JMenuItem("Undo");
      contentArea.undoItem = undoItem;
      undoItem.setMnemonic('U');
      undoItem.setAccelerator(KeyStroke.getKeyStroke("control Z"));
      undoItem.addActionListener(contentArea.undo);
      editMenu.add(undoItem);
      // Redo
      JMenuItem redoItem = new JMenuItem("Redo");
      contentArea.redoItem = redoItem;
      redoItem.setMnemonic('E');
      redoItem.setAccelerator(KeyStroke.getKeyStroke("control E"));
      redoItem.addActionListener(contentArea.redo);
      editMenu.add(redoItem);
      // Separator
      editMenu.addSeparator();
      // Cut
      JMenuItem cutItem = new JMenuItem("Cut");
      contentArea.cutItem = cutItem;
      cutItem.setMnemonic('T');
      cutItem.setAccelerator(KeyStroke.getKeyStroke("control X"));
      cutItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { contentArea.cut(); } });
      editMenu.add(cutItem);
      // Copy
      JMenuItem copyItem = new JMenuItem("Copy");
      contentArea.copyItem = copyItem;
      copyItem.setMnemonic('C');
      copyItem.setAccelerator(KeyStroke.getKeyStroke("control C"));
      copyItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { contentArea.copy(); } });
      editMenu.add(copyItem);
      // Paste
      JMenuItem pasteItem = new JMenuItem("Paste");
      contentArea.pasteItem = pasteItem;
      pasteItem.setMnemonic('P');
      pasteItem.setAccelerator(KeyStroke.getKeyStroke("control V"));
      pasteItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { contentArea.paste(); } });
      editMenu.add(pasteItem);
      // Delete
      JMenuItem deleteItem = new JMenuItem("Delete");
      contentArea.deleteItem = deleteItem;
      deleteItem.setMnemonic('L');
      deleteItem.setAccelerator(KeyStroke.getKeyStroke("typed DELETE"));
      deleteItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { contentArea.replaceSelection(""); } });
      editMenu.add(deleteItem);
      // Separator
      editMenu.addSeparator();
      // Find
      JMenuItem findItem = new JMenuItem("Find...");
      findItem.setMnemonic('F');
      findItem.setAccelerator(KeyStroke.getKeyStroke("control F"));
      findItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { new FindDialog(parentWindow, contentArea); } });
      editMenu.add(findItem);
      // Find Next
      JMenuItem findNextItem = new JMenuItem("Find Next");
      contentArea.findNextItem = findNextItem;
      findNextItem.setMnemonic('N');
      findNextItem.setAccelerator(KeyStroke.getKeyStroke("F3"));
      findNextItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
      {
        if(!contentArea.find(contentArea.staticFindString, contentArea.staticMatchCase, contentArea.staticDownDirection))
          JOptionPane.showMessageDialog(parentWindow, "Cannot find specified text");
      } });
      findNextItem.setEnabled(false);
      editMenu.add(findNextItem);
      // Replace
      JMenuItem replaceItem = new JMenuItem("Replace...");
      replaceItem.setMnemonic('R');
      replaceItem.setAccelerator(KeyStroke.getKeyStroke("control R"));
      replaceItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { new ReplaceDialog(parentWindow, contentArea); } });
      editMenu.add(replaceItem);
      // GoTo
      JMenuItem goToItem = new JMenuItem("Go To...");
      goToItem.setMnemonic('G');
      goToItem.setAccelerator(KeyStroke.getKeyStroke("control G"));
      goToItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { new GoToDialog(parentWindow, contentArea); } });
      editMenu.add(goToItem);
      // Separator
      editMenu.addSeparator();
      // Select All
      JMenuItem selectAllItem = new JMenuItem("Select All");
      selectAllItem.setMnemonic('A');
      selectAllItem.setAccelerator(KeyStroke.getKeyStroke("control A"));
      selectAllItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { contentArea.selectAll(); } });
      editMenu.add(selectAllItem);
      // Insert Time/Date
      JMenuItem timeDateItem = new JMenuItem("Insert Time/Date");
      timeDateItem.setMnemonic('D');
      timeDateItem.setAccelerator(KeyStroke.getKeyStroke("F5"));
      timeDateItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { contentArea.insertTimeDate(); } });
      editMenu.add(timeDateItem);
      // Insert Image
      JMenuItem imageItem = new JMenuItem("Insert Image");
      imageItem.setMnemonic('I');
      imageItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { contentArea.insertImage(); } });
      editMenu.add(imageItem);
    editMenu.addMenuListener(new MenuListener()
    {
      public void menuCanceled(MenuEvent event)
      {
        contentArea.undoItem.setEnabled(true);
        contentArea.redoItem.setEnabled(true);
        contentArea.cutItem.setEnabled(true);
        contentArea.copyItem.setEnabled(true);
        contentArea.pasteItem.setEnabled(true);
        contentArea.deleteItem.setEnabled(true);
      }
      public void menuDeselected(MenuEvent event)
      {
        contentArea.undoItem.setEnabled(true);
        contentArea.redoItem.setEnabled(true);
        contentArea.cutItem.setEnabled(true);
        contentArea.copyItem.setEnabled(true);
        contentArea.pasteItem.setEnabled(true);
        contentArea.deleteItem.setEnabled(true);
      }
      public void menuSelected(MenuEvent event)
      {
        contentArea.undoItem.setEnabled(contentArea.undoManager.canUndo());
        contentArea.redoItem.setEnabled(contentArea.undoManager.canRedo());
        boolean isSelection = true;
        if(contentArea.getSelectedText() == null) { isSelection = false; }
        contentArea.cutItem.setEnabled(isSelection);
        contentArea.copyItem.setEnabled(isSelection);
        if(contentArea.systemClipboard.getContents(this) != null && (contentArea.systemClipboard.getContents(this) instanceof TerpClipboardInterface)) contentArea.pasteItem.setEnabled(true);
        else contentArea.pasteItem.setEnabled(false);
        contentArea.deleteItem.setEnabled(isSelection);
      }
    });
    menuBar.add(editMenu);
    // Format Menu
    JMenu formatMenu = new JMenu("Format");
    formatMenu.setMnemonic('O');
      // Word Wrap
      JCheckBoxMenuItem wordWrapItem = new JCheckBoxMenuItem("Word Wrap");
      wordWrapItem.setMnemonic('W');
      contentArea.setWordWrapState(true);
      wordWrapItem.setState(contentArea.isWordWrapEnabled());
      wordWrapItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { contentArea.setWordWrapState(!contentArea.isWordWrapEnabled()); } });
      formatMenu.add(wordWrapItem);
      // Change Font
      JMenuItem changeFontItem = new JMenuItem("Change Font...");
      changeFontItem.setMnemonic('F');
      changeFontItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
      {
        ChangeFontDialog fontChooser = new ChangeFontDialog(parentWindow);
        fontChooser.setVisible(true);
        if(fontChooser.getOption() == JOptionPane.OK_OPTION)
        {
          contentArea.setIgnoreChanges(6);
          try { contentArea.currentDocument.insertString(0, " ", null); } catch(BadLocationException e) { }
          contentArea.setFont(fontChooser.getFont());
          try { contentArea.currentDocument.remove(0, 1); } catch(BadLocationException e) { }
        }
        fontChooser.dispose();
      } });
      formatMenu.add(changeFontItem);
      // Justify
      JMenu justifyMenu = new JMenu("Justification");
        justifyMenu.setMnemonic('J');
        ButtonGroup justifyGroup = new ButtonGroup();
        JRadioButtonMenuItem justifyLeft = new JRadioButtonMenuItem("Left");
        contentArea.justifyLeft = justifyLeft;
        justifyLeft.setMnemonic('L');
        justifyLeft.setSelected(true);
        justifyLeft.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
          { new StyledEditorKit.AlignmentAction("", StyleConstants.ALIGN_LEFT).actionPerformed(event); } });
        justifyGroup.add(justifyLeft);
        justifyMenu.add(justifyLeft);
        JRadioButtonMenuItem justifyCenter = new JRadioButtonMenuItem("Center");
        contentArea.justifyCenter = justifyCenter;
        justifyCenter.setMnemonic('C');
        justifyCenter.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
          { new StyledEditorKit.AlignmentAction("", StyleConstants.ALIGN_CENTER).actionPerformed(event); } });
        justifyGroup.add(justifyCenter);
        justifyMenu.add(justifyCenter);
        JRadioButtonMenuItem justifyRight = new JRadioButtonMenuItem("Right");
        contentArea.justifyRight = justifyRight;
        justifyRight.setMnemonic('R');
        justifyRight.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
          { new StyledEditorKit.AlignmentAction("", StyleConstants.ALIGN_RIGHT).actionPerformed(event); } });
        justifyGroup.add(justifyRight);
        justifyMenu.add(justifyRight);
      formatMenu.add(justifyMenu);
    formatMenu.addMenuListener(new MenuListener()
    {
      public void menuCanceled(MenuEvent event) {}
      public void menuDeselected(MenuEvent event) {}
      public void menuSelected(MenuEvent event)
      {
        switch(StyleConstants.getAlignment(((StyledEditorKit)(contentArea.getEditorKit())).getInputAttributes()))
        {
          case StyleConstants.ALIGN_LEFT:
            contentArea.justifyLeft.setSelected(true);
            break;
          case StyleConstants.ALIGN_CENTER:
            contentArea.justifyCenter.setSelected(true);
            break;
          case StyleConstants.ALIGN_RIGHT:
            contentArea.justifyRight.setSelected(true);
            break;
        } 
      }
    });
    menuBar.add(formatMenu);
    // Help Menu
    JMenu helpMenu = new JMenu("Help");
    helpMenu.setMnemonic('H');
      // About
      JMenuItem aboutItem = new JMenuItem("About 2Pad");
      aboutItem.setMnemonic('A');
      aboutItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
      {
        final JDialog aboutDialog = new JDialog(parentWindow, "About", true);
        aboutDialog.getContentPane().setLayout(new BoxLayout(aboutDialog.getContentPane(), BoxLayout.Y_AXIS));
          JTextField headerText = new JTextField("2Pad Editor\n" + version +
            "\nApril 2002\nUniversity of Maryland, College Park\nSoftware Engineering - CMSC 435 (Atif M. Memon)\n", 28);
          headerText.setBorder(new EmptyBorder(5, 5, 2, 5));
          //Bug in Java?
          //headerText.setHorizontalAlignment(JTextField.CENTER);
          headerText.setEditable(false);
          aboutDialog.getContentPane().add(headerText);
          JPanel peoplePanel = new JPanel();
          peoplePanel.setLayout(new BoxLayout(peoplePanel, BoxLayout.X_AXIS));
            JTextField coderText = new JTextField("Coding Team:\n" + "Paul Donohue\n" + "Michael Goldgeier\n" +
              "Nathan Hillery\n" + "Matthew Gilhool\n" + "Komi Gnatiko\n", 1);
            coderText.setBorder(new EmptyBorder(3, 5, 5, 2));
            coderText.setEditable(false);
            peoplePanel.add(coderText);
            JTextField testerText = new JTextField("Testing Team:\n" + "William Gomes\n" + "Navead Hashemi\n" +
              "Daniel Emminizer\n", 1);
            testerText.setBorder(new EmptyBorder(3, 3, 5, 5));
            testerText.setEditable(false);
            peoplePanel.add(testerText);
          aboutDialog.getContentPane().add(peoplePanel);
        JButton okButton = new JButton("OK");
        okButton.setAlignmentX((float)0.5);
        okButton.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
          { aboutDialog.dispose(); } });
        aboutDialog.getContentPane().add(okButton);
        aboutDialog.pack();
        aboutDialog.setLocationRelativeTo(parentWindow);
        aboutDialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        aboutDialog.setVisible(true);
      } });
      helpMenu.add(aboutItem);
    menuBar.add(helpMenu);

    parentWindow.setJMenuBar(menuBar);
  }

  protected void createPopupMenu()
  {
    JPopupMenu popupMenu = new JPopupMenu();

    // Undo
    JMenuItem undoPopupItem = new JMenuItem("Undo");
    contentArea.undoPopupItem = undoPopupItem;
    undoPopupItem.setMnemonic('U');
    undoPopupItem.addActionListener(contentArea.undo);
    popupMenu.add(undoPopupItem);
    // Redo
    JMenuItem redoPopupItem = new JMenuItem("Redo");
    contentArea.redoPopupItem = redoPopupItem;
    redoPopupItem.setMnemonic('E');
    redoPopupItem.addActionListener(contentArea.redo);
    popupMenu.add(redoPopupItem);
    // Separator
    popupMenu.addSeparator();
    // Cut
    JMenuItem cutPopupItem = new JMenuItem("Cut");
    contentArea.cutPopupItem = cutPopupItem;
    cutPopupItem.setMnemonic('T');
    cutPopupItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
      { contentArea.cut(); } });
    popupMenu.add(cutPopupItem);
    // Copy
    JMenuItem copyPopupItem = new JMenuItem("Copy");
    contentArea.copyPopupItem = copyPopupItem;
    copyPopupItem.setMnemonic('C');
    copyPopupItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
      { contentArea.copy(); } });
    popupMenu.add(copyPopupItem);
    // Paste
    JMenuItem pastePopupItem = new JMenuItem("Paste");
    contentArea.pastePopupItem = pastePopupItem;
    pastePopupItem.setMnemonic('P');
    pastePopupItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
      { contentArea.paste(); } });
    popupMenu.add(pastePopupItem);
    // Delete
    JMenuItem deletePopupItem = new JMenuItem("Delete");
    contentArea.deletePopupItem = deletePopupItem;
    deletePopupItem.setMnemonic('L');
    deletePopupItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
      { contentArea.replaceSelection(""); } });
    popupMenu.add(deletePopupItem);
    // Separator
    popupMenu.addSeparator();
    // Justify
    JMenu justifyPopupMenu = new JMenu("Justification");
      justifyPopupMenu.setMnemonic('J');
      ButtonGroup justifyGroup = new ButtonGroup();
      JRadioButtonMenuItem justifyLeft = new JRadioButtonMenuItem("Left");
      contentArea.popupJustifyLeft = justifyLeft;
      justifyLeft.setMnemonic('L');
      justifyLeft.setSelected(true);
      justifyLeft.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { new StyledEditorKit.AlignmentAction("", StyleConstants.ALIGN_LEFT).actionPerformed(event); } });
      justifyGroup.add(justifyLeft);
      justifyPopupMenu.add(justifyLeft);
      JRadioButtonMenuItem justifyCenter = new JRadioButtonMenuItem("Center");
      contentArea.popupJustifyCenter = justifyCenter;
      justifyCenter.setMnemonic('C');
      justifyCenter.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { new StyledEditorKit.AlignmentAction("", StyleConstants.ALIGN_CENTER).actionPerformed(event); } });
      justifyGroup.add(justifyCenter);
      justifyPopupMenu.add(justifyCenter);
      JRadioButtonMenuItem justifyRight = new JRadioButtonMenuItem("Right");
      contentArea.popupJustifyRight = justifyRight;
      justifyRight.setMnemonic('R');
      justifyRight.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
        { new StyledEditorKit.AlignmentAction("", StyleConstants.ALIGN_RIGHT).actionPerformed(event); } });
      justifyGroup.add(justifyRight);
      justifyPopupMenu.add(justifyRight);
    popupMenu.add(justifyPopupMenu);
    // Separator
    popupMenu.addSeparator();
    // Select All
    JMenuItem selectAllPopupItem = new JMenuItem("Select All");
    selectAllPopupItem.setMnemonic('A');
    selectAllPopupItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
      { contentArea.selectAll(); } });
    popupMenu.add(selectAllPopupItem);
    // Insert Time/Date
    JMenuItem timeDatePopupItem = new JMenuItem("Insert Time/Date");
    timeDatePopupItem.setMnemonic('D');
    timeDatePopupItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
      { contentArea.insertTimeDate(); } });
    popupMenu.add(timeDatePopupItem);
    // Insert Image
    JMenuItem imagePopupItem = new JMenuItem("Insert Image");
    imagePopupItem.setMnemonic('I');
    imagePopupItem.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent event)
      { contentArea.insertImage(); } });
    popupMenu.add(imagePopupItem);

    popupMenu.addPopupMenuListener(new PopupMenuListener()
    {
      public void popupMenuCanceled(PopupMenuEvent event) {}
      public void popupMenuWillBecomeInvisible(PopupMenuEvent event) {}
      public void popupMenuWillBecomeVisible(PopupMenuEvent event)
      {
        contentArea.undoPopupItem.setEnabled(contentArea.undoManager.canUndo());
        contentArea.redoPopupItem.setEnabled(contentArea.undoManager.canRedo());
        boolean isSelection = true;
        if(contentArea.getSelectedText() == null) { isSelection = false; }
        contentArea.cutPopupItem.setEnabled(isSelection);
        contentArea.copyPopupItem.setEnabled(isSelection);
        if(contentArea.systemClipboard.getContents(this) == null || !(contentArea.systemClipboard.getContents(this) instanceof TerpClipboardInterface)) contentArea.pastePopupItem.setEnabled(false);
        else contentArea.pastePopupItem.setEnabled(true);
        contentArea.deletePopupItem.setEnabled(isSelection);
        switch(StyleConstants.getAlignment(((StyledEditorKit)(contentArea.getEditorKit())).getInputAttributes()))
        {
          case StyleConstants.ALIGN_LEFT:
            contentArea.popupJustifyLeft.setSelected(true);
            break;
          case StyleConstants.ALIGN_CENTER:
            contentArea.popupJustifyCenter.setSelected(true);
            break;
          case StyleConstants.ALIGN_RIGHT:
            contentArea.popupJustifyRight.setSelected(true);
            break;
        } 
      }
    });
    contentArea.setPopupMenu(popupMenu);
  }
}
