//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//CMSC435  UMCellEditor (some of the code is adapted from
//                        SharpTools project )
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

import javax.swing.*;
import javax.swing.border.*;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Font;
import java.awt.Component;
import javax.swing.UIManager;
//*************************************************************
//Class: UMCellEditor
//Description: This is the only editor for the UMTableModel
//             because it is the only cell object editor.
//             In the future it will read all of the settings
//             for the cell in the spreadsheet and display them
//             accordingly
//*************************************************************

public class UMCellEditor extends DefaultCellEditor
{

    // JTextField object this editor uses
    private JTextField m_textField;

    //*********************************************************
    // Public Constructor: UMCellEditor
    // Description: creates new SharpCellEditor
    //*********************************************************
    public UMCellEditor(final JTextField textField)
    {
        super(textField);
	this.m_textField = textField;

        /* To account for formula feature only need
         * to override the setValue method in EditorDelegate inner
         * class.
         */
        delegate = new EditorDelegate()
	{
            public void setValue(Object value)
	    {
		if (value instanceof UMCell)
		{
                    UMCell temp = (UMCell)value;
                     /* when editing formula cell
                     * a string representation is displayed
                     */
                    if (temp.isFormula()) {
                        textField.setText("=" + temp.getFormula().toString());
                    }
                    else {

                        //otherwise it is just the normal string conversion
                        textField.setText(temp.getValue().toString());
                        }

		}
		else
		{
                    //empty cells display nothing
                    textField.setText((value == null) ? "" : value.toString());
                }

            }

	    public Object getCellEditorValue() {
		return textField.getText();
	    }
        };
	textField.addActionListener(delegate);
    }

    /** get the component used by this editor
     * @return the JTextField used by this editor
     */
    public JTextField getTextField() { return m_textField; }

    //***********************************************************
    // Public Method: getTableCellEditorComponent
    // Description: implements javax.swing.table.TableCellRenderer
    //              it supposed to initialize the editor for each
    //              individual cell and return it.
    //
    // Parameters:
    //   table      - JTable
    //   value      - value to assign to the cell at row, column
    //   isSelected - true if cell is selected
    //   hasFocus   - true if cell has focus
    //   row        - the row of the cell to render
    //   column     - the column of the cell to render
    //************************************************************
    public Component getTableCellEditorComponent(JTable table,
                                                 Object value,
                                                 boolean isSelected,
                                                 int row,
                                                 int column)
    {
         Object obj = table.getModel().getValueAt(row, column);
         if (obj instanceof UMCell)
         {
             UMCell cell = (UMCell) obj;
             Font font = cell.getFont();
             Color bColor = cell.getBackground();
             Color fColor = cell.getForeground();

             if (font == null)
             {
                 font = table.getFont();
                 cell.setFont(font);
             }
             if (bColor == null)
             {
                 bColor = table.getBackground();
                 cell.setBackground(bColor);
             }
             if (fColor == null)
             {
                 fColor = table.getForeground();
                 cell.setForeground(fColor);
             }
             m_textField.setForeground(fColor);
             m_textField.setBackground(bColor);
             m_textField.setFont(font);
           }
           else
           {
               m_textField.setForeground(table.getForeground());
               m_textField.setBackground(table.getBackground());
               m_textField.setFont(table.getFont());
           }
         setValue(value, isSelected, row, column);
         return m_textField;
    }


    //*******************************************************************
    // Protected Method: setValue
    // Description: Sets the string for the cell being edited
    //              to value.
    //
    // Parameters:
    //       value      - the string value for this cell, if null empty string
    //                    is displayed
    //       hasFocus   - whether the cell has focus or not
    //       isSelected - whether the cell is selected.
    //       row        - cell row
    //       column     - cell column
    //*********************************************************************
    protected void setValue(Object value,
			    boolean isSelected,
			    int row, int column)
    {
         if (value instanceof UMCell)
         {
            //we only care about the value, not the formula string
            UMCell temp = (UMCell)value;

            // getFormula
            if (temp.isFormula())
            {
               m_textField.setText("=" + temp.getFormula().toString());
            }
            else
            {
	      Object data = temp.getValue();
              //value to display in table
              m_textField.setText((data == null) ? "" : data.toString());
            }
         }
         else
             m_textField.setText((value == null) ? "": value.toString());
    }
}

