//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//CMSC435  UMSpreadSheet (some of the code is adapted from
//                        SharpTools project )
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.*;

//**************************************************************
//Class: UMSpreadSheet --
//Description: this is the main class for the project it contains
//             the main start method and the main gue that is
//             loaded on start up.
//
//Disclaimer: Some of the code and and ideas we found online in
//            the ShartTools project.  We thank them for making
//            their code availible.
//*************************************************************
final public class UMSpreadSheet
    extends JFrame implements ListSelectionListener,
                              MouseListener,
                              ActionListener,
                              TerpOfficeAppInterface
{

    protected int m_maxNumPage = 1;

    public boolean isClosed;
    public boolean m_isStandAlone = false;

    // the rest of the components
    private Container m_container;
    private JTable m_table;
    private JScrollPane m_scrollPane;
    private JToolBar m_toolBar;
    private JMenuBar m_menuBar;
    private JPanel m_barPanel;
    private JPanel m_inputPane;
    public  JToggleButton boldButton;
    public  JToggleButton italicButton;

    static private Config m_config;

    private UMTableModel m_tableModel;
    private ListSelectionModel m_rowSelectionModel;
    private ListSelectionModel m_columnSelectionModel;

    // these are used to access our global objects
    static public Config getConfig() { return m_config; }

    public JTable getTable() { return m_table; }
    public UMTableModel getTableModel() { return m_tableModel; }
    public JToolBar getToolBar() { return m_toolBar; }
    public JMenuBar getJMenuBar() { return m_menuBar; }


    private FileMenuOp m_fileMenuOp;
    private EditMenuOp m_editMenuOp;
    private FormatMenuOp m_formatMenuOp;
    private FunctionsMenuOp m_functionsMenuOp;

    // initialize JMenu fields
    // menu headings
    private JMenu m_fileMenu;
    private JMenu m_editMenu;
    private JMenu m_formatMenu;
    private JMenu m_functionsMenu;
    private PopupMenu m_popup;

    // Tool bar selections
    public JComboBox m_fontList;
    public JComboBox m_fontSizeList;

    // recources
    final private ImageIcon newIcon = getImageIcon("new.gif");
    final private ImageIcon openIcon = getImageIcon("open.gif");
    final private ImageIcon saveIcon = getImageIcon("save.gif");
    final private ImageIcon printIcon = getImageIcon("print.gif");
    final private ImageIcon cutIcon = getImageIcon("cut.gif");
    final private ImageIcon copyIcon = getImageIcon("copy.gif");
    final private ImageIcon pasteIcon = getImageIcon("paste.gif");
    final private ImageIcon boldIcon = getImageIcon("bold.gif");
    final private ImageIcon italicIcon = getImageIcon("italic.gif");


    public static int m_baseRow = 0;
    public static int m_baseCol = 1;
    public static int m_maxRow = 49;
    public static int m_maxCol = 50;

    //******************************************************************
    // UMSpreadSheet
    // Public constructor to create a new spreadsheet
    //*****************************************************************
    public UMSpreadSheet() {

        super("University of Maryland CMSC435 Spreadsheet");

        //int x = config.getInt("X");
	//int y = config.getInt("Y");

        int x = 50;
        int y = 50;
	Dimension scrdim = getToolkit().getScreenSize();

	// make sure the point is valid
	if (x>=0 && y>=0 &&
	    x<(int)scrdim.getWidth() && y <(int)scrdim.getHeight())
	    setLocation(x, y);

	setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        // get content pane
	m_container = this.getContentPane();
	m_container.setLayout(new BorderLayout());

        // Create a pane for input bar and toolbar
        m_inputPane = new JPanel();
        m_inputPane.setLayout(new BorderLayout());

        //m_inputBar = new JTextField();
	// create the table
	m_table = new JTable(m_maxRow + 1,m_maxCol + 1);
        m_table.setGridColor(Color.lightGray);

	// create the Fonts combobox in the table
        //START
        java.awt.GraphicsEnvironment ge =
            java.awt.GraphicsEnvironment.getLocalGraphicsEnvironment();
        String[] fontNames = ge.getAvailableFontFamilyNames();

        m_fontList = new JComboBox();
        m_fontList.addItem("[Automatic]");
        for (int i = 0; i < fontNames.length; i++)
            m_fontList.addItem(fontNames[i]);

        m_fontList.setBackground(Color.white);
        m_fontList.setSelectedIndex(1);
        m_fontList.addItemListener(new ItemListener() {
                public void itemStateChanged(ItemEvent e)
                {
                    if (e.getStateChange() == ItemEvent.SELECTED)
                        m_formatMenuOp.setFontOp(e.getItem());
                }
            });

        String [] fontSizes = { "-", "8", "9", "10", "11", "12", "14",
                               "16","18", "20", "24", "26", "28" ,"36", "48",
                               "72"};

        m_fontSizeList = new JComboBox(fontSizes);
        m_fontSizeList.setBackground(Color.white);
        m_fontSizeList.setSelectedIndex(5);
        m_fontSizeList.addItemListener(new ItemListener() {
                public void itemStateChanged(ItemEvent e)
                {
                    if (e.getStateChange() == ItemEvent.SELECTED)
                        m_formatMenuOp.setFontSizeOp(e.getItem());
                }
            });
        try
        {
            m_table.setFont(new Font(fontNames[0], Font.PLAIN,
                                 Integer.parseInt(fontSizes[5])));
        }catch (Exception fontE)
        {
            m_table.setFont(new Font(fontNames[0], Font.PLAIN,
                                     12));
        }
        //END

	// set up fileMenu
        m_fileMenu = new JMenu("File", true);
	m_fileMenu.setMnemonic(KeyEvent.VK_F); //used constructor instead
	// add fileMenu actions

        // New
	JMenuItem menuItem = new JMenuItem("New...");
	menuItem.setMnemonic(KeyEvent.VK_N);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_N, ActionEvent.CTRL_MASK));
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_fileMenuOp.newFile();
		}
	    });
	m_fileMenu.add(menuItem);

	// Save
        menuItem = new JMenuItem("Save");
	menuItem.setMnemonic(KeyEvent.VK_S);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_S, ActionEvent.CTRL_MASK));
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_fileMenuOp.saveFile();
		}
	    });
	m_fileMenu.add(menuItem);

        // Save As
        menuItem = new JMenuItem("Save As...");
	menuItem.setMnemonic(KeyEvent.VK_S);
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_fileMenuOp.saveAsFile();
		}
	    });
	m_fileMenu.add(menuItem);
        // Open
        menuItem = new JMenuItem("Open");
	menuItem.setMnemonic(KeyEvent.VK_O);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_O, ActionEvent.CTRL_MASK));
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_fileMenuOp.openFile();
		}
	    });
	m_fileMenu.add(menuItem);
        m_fileMenu.addSeparator();
        // Print
        menuItem = new JMenuItem("Print...");
	menuItem.setMnemonic(KeyEvent.VK_P);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_P, ActionEvent.CTRL_MASK));
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_fileMenuOp.printFile();
		}
	    });
	m_fileMenu.add(menuItem);
        // Exit
        m_fileMenu.addSeparator();
        menuItem = new JMenuItem("Exit");
	menuItem.setMnemonic(KeyEvent.VK_E);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_E, ActionEvent.CTRL_MASK));
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_fileMenuOp.exitFile(2);
		}
	    });
	m_fileMenu.add(menuItem);


	//Set up Edit menu
	m_editMenu = new JMenu("Edit", true);
	m_editMenu.setMnemonic(KeyEvent.VK_E);

        // Cut
        menuItem = new JMenuItem("Cut");
	menuItem.setMnemonic(KeyEvent.VK_X);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_X, ActionEvent.CTRL_MASK));
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_editMenuOp.cutOp();
		}
	    });
	m_editMenu.add(menuItem);
        // Copy
        menuItem = new JMenuItem("Copy");
	menuItem.setMnemonic(KeyEvent.VK_C);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_C, ActionEvent.CTRL_MASK));
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_editMenuOp.copyOp();
		}
	    });
	m_editMenu.add(menuItem);
        // Paste
        menuItem = new JMenuItem("Paste");
	menuItem.setMnemonic(KeyEvent.VK_P);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_P, ActionEvent.CTRL_MASK));
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_editMenuOp.pasteOp();
		}
	    });
	m_editMenu.add(menuItem);

        // Clear
        menuItem = new JMenuItem("Clear");
	menuItem.setMnemonic(KeyEvent.VK_L);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_DELETE, 0, true));
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_editMenuOp.clearOp();
		}
	    });
	m_editMenu.add(menuItem);

        //Set up Format Menu
	m_formatMenu = new JMenu("Format", true);
	m_formatMenu.setMnemonic(KeyEvent.VK_F);

        // Set column width
        menuItem = new JMenuItem("Column Width");
	menuItem.setMnemonic(KeyEvent.VK_W);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_W, ActionEvent.CTRL_MASK));
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_formatMenuOp.columnWidthOp();
		}
	    });
	m_formatMenu.add(menuItem);
        // Row Height
        menuItem = new JMenuItem("Row Height");
	menuItem.setMnemonic(KeyEvent.VK_H);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_H, ActionEvent.CTRL_MASK));
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_formatMenuOp.rowHeightOp();
		}
	    });
        m_formatMenu.add(menuItem);
        // Format Cells
        m_formatMenu.addSeparator();
        menuItem = new JMenuItem("Format Cells...");
	menuItem.setMnemonic(KeyEvent.VK_E);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_E, ActionEvent.CTRL_MASK));
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_formatMenuOp.formatCellsOp();
		}
	    });
	m_formatMenu.add(menuItem);

        //Set up Functions Menu
	m_functionsMenu = new JMenu("Functions", true);
	m_functionsMenu.setMnemonic(KeyEvent.VK_U);

        // Sum
        menuItem = new JMenuItem("Sum");
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_functionsMenuOp.sumOp();
		}
	    });
	m_functionsMenu.add(menuItem);

        // Average
         menuItem = new JMenuItem("Average");
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_functionsMenuOp.averageOp();
		}
	    });
	m_functionsMenu.add(menuItem);
        // Max
        menuItem = new JMenuItem("Max");
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_functionsMenuOp.maxOp();
		}
	    });
	m_functionsMenu.add(menuItem);

        // Min
         menuItem = new JMenuItem("Min");
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_functionsMenuOp.minOp();
		}
	    });
	m_functionsMenu.add(menuItem);

        // Square Root
        menuItem = new JMenuItem("Square Root");
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_functionsMenuOp.sqrtOp();
		}
	    });
	m_functionsMenu.add(menuItem);

        // Sine
         menuItem = new JMenuItem("Sine");
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_functionsMenuOp.sineOp();
		}
	    });
	m_functionsMenu.add(menuItem);

        // Count
        menuItem = new JMenuItem("Count");
	menuItem.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_functionsMenuOp.countOp();
		}
	    });
	m_functionsMenu.add(menuItem);

        //*******Popup Menu Method******

        m_popup = new PopupMenu("Edit");
        MenuItem mi;

        mi = new MenuItem("Format Cells...");
        mi.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
                    System.out.println("IN OP");
		    m_formatMenuOp.formatCellsOp();
                    m_table.requestDefaultFocus();
		}
	    });
        m_popup.add(mi);
        m_popup.addSeparator();

        mi = new MenuItem("Cut");
        mi.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_editMenuOp.cutOp();
                    m_table.requestDefaultFocus();
		}
	    });
        m_popup.add(mi);
        mi = new MenuItem("Copy");
        mi.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
        	    m_editMenuOp.copyOp();
                    m_table.requestDefaultFocus();
		}
	    });
        m_popup.add(mi);

        mi = new MenuItem("Paste");
        mi.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
        	    m_editMenuOp.pasteOp();
                    m_table.requestDefaultFocus();
		}
	    });
        m_popup.add(mi);

        mi = new MenuItem("Clear");
        mi.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
        	    m_editMenuOp.clearOp();
                    m_table.requestDefaultFocus();
		}
	    });
        m_popup.add(mi);



        m_table.add(m_popup);
        m_table.addMouseListener(this);

	// set up menu bar and menus
        m_menuBar = new JMenuBar();

        m_menuBar.add(m_fileMenu);
        m_menuBar.add(m_editMenu);
        m_menuBar.add(m_formatMenu);
	m_menuBar.add(m_functionsMenu);

        // button initializations
        JButton newButton = new JButton(newIcon);
	newButton.setToolTipText("New");
	newButton.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_fileMenuOp.newFile();
		    m_table.requestFocus();
		}
	    });
        JButton openButton = new JButton(openIcon);
	openButton.setToolTipText("Open");
	openButton.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_fileMenuOp.openFile();
		    m_table.requestFocus();
		}
	    });

	JButton saveButton = new JButton(saveIcon);
	saveButton.setToolTipText("Save");
	saveButton.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_fileMenuOp.saveFile();
		    m_table.requestFocus();
		}
	    });


	JButton printButton = new JButton(printIcon);
	printButton.setToolTipText("Print");
	printButton.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
                    m_fileMenuOp.printFile();
                    m_table.requestFocus();
		}
	    });

        italicButton = new JToggleButton(italicIcon);
        italicButton.setToolTipText("Italic");
        italicButton.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_formatMenuOp.setItalicOp(italicButton.isSelected());
		    m_table.requestFocus();
		}
	    });

        boldButton = new JToggleButton(boldIcon);
        boldButton.setToolTipText("Bold");
        boldButton.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
                    m_formatMenuOp.setBoldOp(boldButton.isSelected());
		    m_table.requestFocus();
		}
	    });
        JButton cutButton = new JButton(cutIcon);
	cutButton.setToolTipText("Cut");
	cutButton.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_editMenuOp.cutOp();
		    m_table.requestFocus();
		}
	    });
        JButton copyButton = new JButton(copyIcon);
	copyButton.setToolTipText("Copy");
	copyButton.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_editMenuOp.copyOp();
		    m_table.requestFocus();
		}
	    });

        JButton pasteButton = new JButton(pasteIcon);
	pasteButton.setToolTipText("Paste");
        pasteButton.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
		    m_editMenuOp.pasteOp();
		    m_table.requestFocus();
		}
	    });



        /*
	 * Set up toolbar
	 *
	 * Toolbar can be customized in the configuration file.
	 */
        m_toolBar = new JToolBar();
	m_toolBar.setFloatable(false);

	m_toolBar.add(newButton);
        m_toolBar.add(openButton);
        m_toolBar.add(saveButton);
	m_toolBar.addSeparator();
        m_toolBar.add(printButton);
        m_toolBar.addSeparator();
        m_toolBar.add(cutButton);
        m_toolBar.add(copyButton);
        m_toolBar.add(pasteButton);
	m_toolBar.addSeparator();

        JPanel fontPanel = new JPanel();
        fontPanel.add(m_fontList);
        fontPanel.add(m_fontSizeList);

        m_toolBar.add(fontPanel);
        m_toolBar.addSeparator();
	m_toolBar.add(italicButton);
        m_toolBar.add(boldButton);
        m_toolBar.addSeparator();

        newTableModel(m_maxCol + 1,m_maxRow+1);
	// set window pos and size
	int w = 600;
        int h = 500;
	if (w >= 0 && h >= 0)
	    m_table.setPreferredScrollableViewportSize(new Dimension(w, h));

	// init Operations objects
        // Change later to save clipboard info
        m_editMenuOp = new EditMenuOp(this);
	m_fileMenuOp = new FileMenuOp(this);
        m_formatMenuOp = new FormatMenuOp(this);
        m_functionsMenuOp = new FunctionsMenuOp(this);

	// clobber resizing of all columns
	m_table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

        // set table editor and renderer to custom ones DO THIS LATER
	m_table.setDefaultRenderer(UMCell.class, new UMCellRenderer());
        m_table.setDefaultEditor(UMCell.class, new UMCellEditor(
						  new JTextField()));

	// set selection mode for contiguous  intervals
        m_table.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        m_table.setCellSelectionEnabled(true);

	// we don't allow reordering
        m_table.getTableHeader().setReorderingAllowed(false);
	m_table.getTableHeader().addMouseListener(new HeaderMouseAdapter());


	// create selection models
	m_rowSelectionModel = m_table.getSelectionModel();
        m_columnSelectionModel = m_table.getColumnModel().getSelectionModel();

	// add selection listeners to the selection models
	m_rowSelectionModel.addListSelectionListener(this);
	m_columnSelectionModel.addListSelectionListener(this);


	// set menu bar
	setJMenuBar(m_menuBar);
        // Set toolbar and inputBar on one Pane
        m_inputPane.add(m_toolBar, BorderLayout.NORTH);
        //m_inputPane.add(m_inputBar, BorderLayout.SOUTH);
	m_container.add(m_inputPane, BorderLayout.NORTH);

	m_scrollPane = new JScrollPane(m_table,
				     JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				     JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);

        m_container.add(m_scrollPane, BorderLayout.CENTER);

        // add window exit listener
        addWindowListener (new WindowAdapter () {
		public void windowOpened (WindowEvent evt) {
		    m_table.requestFocus();
		}

		public void windowClosing (WindowEvent evt) {
		    exit();
		}
	    });

	// initial selection
	resetSelection();

	m_table.setRequestFocusEnabled(true);
	m_menuBar.setRequestFocusEnabled(false);
	m_toolBar.setRequestFocusEnabled(false);
        //m_inputBar.setRequestFocusEnabled(true);
	m_table.requestFocus();

	pack();
    }

    //**************************************************************
    // setColumnWidth
    // Description: sets the widht  of the columns
    //*************************************************************
    public void setColumnWidth(int colWidth) {

	if (colWidth > 0)
	    for (int i = m_baseCol; i < m_tableModel.getColumnCount(); i ++) {
		TableColumn column = m_table.getColumnModel().getColumn(i);
		column.setMinWidth(colWidth);
		column.setPreferredWidth(colWidth);
	    }
    }

    //*****************************************************************
    // newTableModel
    // Description: Creates new blank SharpTableModel object with 
    //              specified number of rows and columns.  
    //             
    //  table is set to this table model to update screen.
    //
    // @param rows number of rows in new table model
    // @param cols number of columns in new table model
    //****************************************************************
    public void newTableModel(int rows, int cols) {
	m_tableModel = new UMTableModel(this, rows, cols);
	m_table.setModel(m_tableModel);

	setBaseColumnWidth();
        setColumnWidth(80);

	m_tableModel.setModified(false);
	resetSelection();
	m_table.requestFocus();
    }

    //*****************************************************************
    // openInitFile
    // Description: opens a spread sheet with a file
    //              used only in main
    //*****************************************************************
    public void openInitFile(String name)
    {
        m_fileMenuOp.openTableModel(new File(name));
    }
    //*********************************************************
    // exit
    // Descripiton: exit the application
    //*********************************************************
    public void exit()
    {
	m_fileMenuOp.exitFile(1);
    }

    //*********************************************************
    // checkSaveState
    // Descripiton: Check the save menu/button state
    //              Enable only when the file has been modified
    //*********************************************************
    public void checkSaveState() {
	boolean modified = m_tableModel.isModified();
	//saveMenuItem.setEnabled(modified);
	//saveButton.setEnabled(modified);
    }

    //**************************************************************
    // setBaseColumnWidth
    // Description: intializes the width for the columns
    //*************************************************************
    public void setBaseColumnWidth() {
	// resize first column
	if (m_baseCol > 0) {
	    TableColumn firstColumn =
                m_table.getColumnModel().getColumn(m_baseCol - 1);
	    int firstColWidth = 25;
	    if (firstColWidth>0) {
		firstColumn.setMinWidth(firstColWidth);
		firstColumn.setPreferredWidth(firstColWidth);
	    }
	}
    }

    //**************************************************************
    // main
    // Description: starts up the spreadsheet as a standalone
    //              aplication
    //*************************************************************
    public static void main(String args[]){
	// read configuration file
	m_config = new Config("umspreadsheet.ini");

        Debug.setDebug(false);
        

        UMSpreadSheet spreadsheet = new UMSpreadSheet();
        spreadsheet.m_isStandAlone = true;
        spreadsheet.show();
	if (args.length>0){
          //System.out.println("----> "+args[0]);
	    spreadsheet.openInitFile(args[0]);
        }

	Formula.registerFunctions(); 
    }

    //**************************************************************
    // updateOps
    // Description: should be used to updated the operations if table
    //              model chages
    //*************************************************************
    public void updateOps()
    {
        m_formatMenuOp.updateTableModel();
    }

    //****************************************************************
    //ImageIcon
    //Description: this is a static function to help loading images
    //***************************************************************
    public static ImageIcon getImageIcon(String name) {
	URL url = ClassLoader.getSystemResource(name);
	if (url == null) {
	    System.out.println("image "+name+" not found");
	    return null;
	}
	return new ImageIcon(url);
    }

    // INTERFACE: ListSelectionListener
    //**************************************************************
    // valueChanged
    // Description: sets the seleciton or the table, aslo 
    //              makes sure that the shorcut formating gue is update
    //              for the new selection.
    //*************************************************************
    public void valueChanged(ListSelectionEvent e) {
	m_table.requestFocus();

	// Ignore extra messages
	if (e.getValueIsAdjusting()) return;

	// Get event source
	ListSelectionModel lsm = (ListSelectionModel)e.getSource();

	if (lsm.isSelectionEmpty()) {
	    // always set selection
	    m_table.setRowSelectionInterval(m_baseRow, m_baseRow);
	    m_table.setColumnSelectionInterval(m_baseCol, m_baseCol);
	} else {
	    // Get selected column
	    if (m_table.getSelectedColumn() < m_baseCol) {
		int columnCount = m_table.getColumnCount();
		m_table.setColumnSelectionInterval(m_baseCol, columnCount - 1);
		m_table.removeColumnSelectionInterval(m_baseRow,m_baseRow);
	    }
	}


        // Get the state of cells, and be ready to reset the gue
        CellRange cells = m_tableModel.getSelection();
        boolean italic = m_formatMenuOp.isItalic(cells);
        boolean bold = m_formatMenuOp.isBold(cells);
        String fontSize = m_formatMenuOp.getFontSize(cells);
        String fontName = m_formatMenuOp.getFontName(cells);

        boldButton.setSelected(bold);
        italicButton.setSelected(italic);

        // Declare final stuff to use in the later funciton
        final String fFontName = fontName;
        final String fFontSize = fontSize;

        // Set this on event dispach thread
        SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    for (int i = 0; i < m_fontList.getItemCount(); i++)
                        {
                            //System.out.println(
                            //   (String)m_fontList.getItemAt(i) +
                            //   " " + fFontName);

                            if (((String)m_fontList.getItemAt(i)
                                 ).compareTo(fFontName) == 0)
                                {
                                    m_fontList.setSelectedIndex(i);
                                    break;
                                }
                        }
                    for (int i = 0; i < m_fontSizeList.getItemCount(); i++)
                        {
                            if (((String)m_fontSizeList.getItemAt(i)
                                 ).compareTo(fFontSize) == 0)
                                {
                                    m_fontSizeList.setSelectedIndex(i);
                                    break;
                                }
                        }
                }});

    }

    //**************************************************************
    // resetSelection
    // Description: sets the selection to first row, first col
    //*************************************************************
    public void resetSelection() {
    	m_table.setRowSelectionInterval(m_baseRow, m_baseRow);
	m_table.setColumnSelectionInterval(m_baseCol, m_baseCol);
    }

    // INTEFACE: MouseAdapter
    public void actionPerformed(ActionEvent e) {}
    public void mouseEntered(MouseEvent e){}
    public void mouseExited(MouseEvent e){}
    public void mouseReleased(MouseEvent e){}
    public void mousePressed(MouseEvent e) {}

    //**************************************************************
    // mouseClicked
    // Description: displays the popup menu
    //*************************************************************
    public void mouseClicked(MouseEvent e)
    {
        if (e.getModifiers() == InputEvent.BUTTON3_MASK)
        {
            m_popup.show(e.getComponent(), e.getX(), e.getY());
        }
    }

    //******************************************************************
    //Class: HeaderMouseAdapter
    //Description:  This class is the mouse listener for the table
    //              it is responsible for all of the selections
    //*****************************************************************

    class HeaderMouseAdapter extends MouseAdapter {

	public void mouseClicked(MouseEvent e) {
	    TableColumnModel colModel =
		m_table.getColumnModel();
	    int col =
		colModel.getColumn(colModel.getColumnIndexAtX(
				    e.getX())).getModelIndex();

	    int rowCount = m_table.getRowCount();
	    m_table.setRowSelectionInterval(m_baseRow, rowCount - 1);

	    if (col < m_baseCol)
		m_table.setColumnSelectionInterval(m_baseCol,
						 m_table.getColumnCount()-1);
	    else
		m_table.setColumnSelectionInterval(col, col);
	}
    }

    // INTEFACE: TerpOffice
    //**************************************************************
    // close
    // Description: used to exit the application without system.exit
    //*************************************************************
    public void close()
    {
        if (alive())
            exit();
      //TerpOffice.removeApps(this);
    }

    //**************************************************************
    // launch
    // Description: starts up the spreadsheet from within an office
    //*************************************************************
    public void launch(File file)
    {
    	// read configuration file
	//m_config = new Config("umspreadsheet.ini");

        Debug.setDebug(false);

        //UMSpreadSheet spreadsheet = new UMSpreadSheet();
        this.show();
        if (file != null)
            m_fileMenuOp.openTableModel(file);
	Formula.registerFunctions();
    }
    //**************************************************************
    // alive
    // Description: returns true if spreadsheet window has not beed
    //              disposed yet
    //*************************************************************
    public boolean alive()
    {
      return isDisplayable();
    }
}



