//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//CMSC435  UMTableModel (some of the code is adapted from
//                        SharpTools project )
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
import java.util.*;
import java.awt.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableColumnModelEvent;


//**************************************************************
//Class: UMTableModel --
//Description: this is the main backend class for the spreadsheet
//             it contains the model of our table.
//Disclaimer: Some of the code and and ideas we found online in
//            the ShartTools project.  We thank them for making
//            their code availible.
//*************************************************************


public class UMTableModel extends DefaultTableModel {
    // Stores modified state of document
    private boolean m_modified;


    /**  Stores file name of current document */
    //    private File file;

    // This the actual spread sheet application
    private UMSpreadSheet m_spreadsheet;

    private String Fontlist[]=GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
    //******************************************************
    //Public Constructor: UMTableModel
    //Description: This is the default constructor, that
    //             just sets the reference to the application
    //*******************************************************
    public UMTableModel(UMSpreadSheet spreadsheet)
    {
        super();

	// initialize state to unmodified and file to untitled
	m_modified = false;
	this.m_spreadsheet = spreadsheet;
    }

    //******************************************************
    //Public Constructor: UMTableModel
    //Description: This constructor creates an empty table
    //             with numRow rows and numCol columns    //             including the row and column label row and column.
    //
    //             The zeroth column are headers. The rows
    //             are numbered 1, 2, ... and columns are
    //             labeled A, B, ... . Thus, it really holds
    //             numRow X numCol-1 of data. The table is
    //             initialized with no cell objects. Each
    //             coordinate that will have data will have a cell object
    //             created later.
    // Parameters:
    //       spreadsheet - gui object to associated with this UMTableModel
    //       numRows     - total number of rows including row header
    //       numColumns  - total number of columns including column header
    //************************************************************

    public UMTableModel(UMSpreadSheet spreadsheet,
			int numRows,
			int numColumns)
    {

        super(numRows, numColumns+1);
        //initialize row headers
	for(int row = UMSpreadSheet.m_baseRow; row < numRows; row++)
	{
            UMCell temp = new UMCell(new Integer(row+1));
            super.setValueAt(temp, row, 0);
        }

        //initialize all the cells
        for(int row = UMSpreadSheet.m_baseRow; row < numRows; row++)
	    for (int col = UMSpreadSheet.m_baseCol; col < numColumns+1; col++)
		// we initialize it here
		super.setValueAt(new UMCell(""), row, col);

        // initialize state to unmodified and file to untitled
	m_modified = false;
	this.m_spreadsheet = spreadsheet;
    }

    //***********************************************************
    //Public Constructor: UMTableModel
    //Description: This constructor if for the convinience
    //             for loading objects that are already in an
    //             array.
    // Parameters:
    //       spreadsheet - gui object to associated with this UMTableModel
    //       data        - array of object to place into the UMTableModel
    //************************************************************

    public UMTableModel(UMSpreadSheet spreadsheet, Object[][] data)
    {

        this(spreadsheet, spreadsheet.m_maxRow + 1,
              spreadsheet.m_maxCol + 1);

        /* load the data */
        for(int i = UMSpreadSheet.m_baseRow; i < data.length &&
             i <= spreadsheet.m_maxRow; i++)
        {
            for(int j = UMSpreadSheet.m_baseCol; j < data[i].length &&
                j <= spreadsheet.m_maxCol; j++)
	    {
                doSetValueAt(data[i][j], i, j);
            }
        }

        // initialize state to unmodified and file to untitled
	m_modified = false;
	//	file = new File("Untitled");
	this.m_spreadsheet = spreadsheet;
    }

    //***********************************************************
    // Public Method: getCellAt
    // Description:  This method gets the whole Cell object located
    //               at the these coordinates. This method avoids
    //               the casting required when using getValueAt.
    //
    //  If the coordinates specify a cell that is out of bounds then
    //  it returns null. If a cell does not exist in the SharpTableModel
    //  at that valid coordinate, it creates an empty cell, places it at
    //  that spot and returns it.
    //
    // Parameters:
    //      aRow - the row of the cell
    //      aColumn - the column of the cell
    //************************************************************

    public UMCell getCellAt(int aRow, int aColumn)
    {
        /* check for out of bounds */
	if (aRow < 0 || aRow >= getRowCount() ||
	    aColumn < 0 || aColumn >= getColumnCount()) {
	    return null;
	}

        UMCell temp = (UMCell)super.getValueAt(aRow, aColumn);
	return temp;
    }


    //************************************************************
    //Public Method: getValueAt
    //Description: This class returns the cell object at those
    //             coordinates. It does  exactly the same thing as
    //             getCellAt except that the return type is Object.
    //
    //   It is implemented because TableModel requires this method
    //   return an Object.
    //
    // Parameters:
    //      aRow     - the row coordinate
    //      aColumn  - the column coordinate
    //*************************************************************
    public Object getValueAt(int aRow, int aColumn)
    {
        return getCellAt(aRow, aColumn);
    }

    //************************************************************
    //Public Method: setValueAt
    //Description: Sets value of the cell.  It should take car of
    //             formulas and data.
    //
    // This function is called by JTable automatically, which means
    // a user has to manually input something.
    //
    // Parameters:
    //      aValue     - the data to set cell to.
    //      aRow       - row coordinate
    //      aColumn    - column coordinate
    //*************************************************************
    public void setValueAt(Object aValue, int aRow, int aColumn)
    {
	doSetValueAt(aValue, aRow, aColumn);
    }

    //************************************************************
    //Public Method: doSetValueAt
    //Description: this method does the actal work for setting values
    // Parameters:
    //      aValue     - the data to set cell to.
    //      aRow       - row coordinate
    //      aColumn    - column coordinate
    //*************************************************************
    public void doSetValueAt(Object aValue, int aRow, int aColumn)
    {
      m_modified = true;
	if (aValue == null)
	    aValue = new String("");
	if (aValue instanceof String)
	{
	    String input = (String)aValue;
            /* try making it a formula */
	    if (input.startsWith("=")) {
		Formula form = null;

		//create formula and its value and put in a cell
		try {
		    form = new Formula(input.substring(1),
				       aRow, aColumn);
		    setCellAt(form, aRow, aColumn);

		}
		catch (ParserException e) {
		    // no parsing
		    form = new Formula(input.substring(1), aRow, aColumn, e);
		    setCellAt(form, aRow, aColumn);
		    getCellAt(aRow, aColumn).setValue(e);
		}
            }
            else {

               try{
		  Float data = new Float(input);
		  setCellAt(data, aRow, aColumn);
	        }catch (NumberFormatException e2) {
		/* all else fails treat as string */
		  setCellAt(aValue, aRow, aColumn);
                }

            }
            }
        else {
            /* it is an object, assume it is exactly
             * what should the cell be set to
             */
            if (aValue instanceof Formula) {
                try {
		    Formula form = (Formula)aValue;
		    setCellAt(new Formula(form, aRow, aColumn), aRow, aColumn);
                }catch (ParserException e) {
	            //	    errorMessage(e.toString());
                    Formula form2 = (Formula)aValue;
                    form2 = new Formula(form2.toString(), aRow, aColumn, e);
		    setCellAt(form2, aRow, aColumn);
		    getCellAt(aRow, aColumn).setValue(e);
                }
            }else {
                setCellAt(aValue, aRow, aColumn);
            }
        }

	/* it is an object, assume it is exactly
	 * what should the cell be set to
	 */
        // ADD this later to see if it is already of good object like formula

    }

    public boolean isCellEditable(int row, int column)
    {
	return column != 0;
    }

    //*********************************************************
    // Public Method: getColumnClass
    // Description:   JTable uses this method to determine the
    //                default renderer and editor for each cell.
    //                This method tells JTable to use
    //                UMCellRender and UMCellEditor.
    //
    // Parameters:
    //      c    -  the column for which we need to determine the class
    //************************************************************
    public Class getColumnClass(int c)
    {
        /* only cell objects in this TableModel */
        return UMCell.class;
    }


    //*******************************************************
    // Public Method: setModified
    // Description:  Sets modified state of current document
    //
    // Parameters:
    //      modified - true sets state to modified
    //********************************************************
    public void setModified(boolean modified)
    {
	this.m_modified = modified;
	// enable/disable the "Save" button
    }

    //*******************************************************
    // Public Method: isModified
    // Description: Returns modified state of the document
    //********************************************************
    public boolean isModified()
    {
	return m_modified;
    }

    //*******************************************************
    // Public Method: getTable
    // Description: Returns JTable
    //*******************************************************
    public JTable getTable() {
	return m_spreadsheet.getTable();
    }

    //*********************************************************
    // Public Method: isEmptyCell
    // Description: returns true if the cell in the table is empty
    //*********************************************************
    public boolean isEmptyCell(int row, int col)
    {
	return getCellAt(row, col).getValue().equals("");
    }


    //*********************************************************
    // Public Method: getColumnName
    // Description: this the the reason we had our columns messed
    //              up.
    //*********************************************************
     public String getColumnName(int col) {
         //Debug.println("name of column "+col);
	if (col < UMSpreadSheet.m_baseCol)
	    return "";
	else
        {
	    String str = String.valueOf(Node.translateColumn(col));
            // Debug.println(Node.translateColumn(str) + " " +
            //                   str + "" + col);
            return str;
        }
    }
    //*********************************************************
    // Public Method: getSelection
    // Description: this returns a cell range that is a current
    //              selection
    //*********************************************************
    public CellRange getSelection()
    {
        int ux = m_spreadsheet.m_baseCol ,
            uy = m_spreadsheet.m_baseRow ,
            lx = m_spreadsheet.m_baseCol,
            ly = m_spreadsheet.m_baseRow;
        JTable table = m_spreadsheet.getTable();

        if (table.getSelectedColumn() != -1)
        {
            ux  = table.getSelectedColumn();
            lx  = ux + table.getSelectedColumnCount() - 1;
        }
        if (table.getSelectedRow() != -1)
        {
            uy = table.getSelectedRow();
            ly = uy + table.getSelectedRowCount() - 1;
        }


        return new CellRange(uy,ly, ux,lx, this);
    }
    //*********************************************************
    // Public Method: clearRange
    // Description: Clears a range of cells
    //*******************************************************
    public void clearRange(CellRange range)
    {
      fill(range,null);
    }
    //*********************************************************
    // Protected Method: fill
    // Description: Fills a range of cells with data in Object data
    //*******************************************************
    protected void fill(CellRange range, Object data){
      for(int row=range.getStartRow(); row<=range.getEndRow();row++)
      {
        for(int col=range.getStartCol();col<= range.getEndCol();col++)
        {
          doSetValueAt(data,row,col);
        }
      }
    }

    public Number getNumericValueAt(int row, int col) throws ParserException {
        UMCell cell = getCellAt(row, col);
	if (cell != null) {
	    int type = cell.getType();
	    if (type == UMCell.FORMULA) {
		Object value = cell.getValue();
		Formula form = cell.getFormula();
		// if need recalc
		if (form.needsRecalc()) {
		    try {
			value = form.evaluate(this, row, col);
			cell.setValue(value);
		    }
		    catch (ParserException e) {
			cell.setValue(e);
			value = e;
		    }
		}

		if (value instanceof ParserException)
		    throw (ParserException)value;
		else
		    return (Number)cell.getValue();
	    }
	    else if (type == UMCell.NUMBER)
		return (Number)cell.getValue();
	    else
		return new Float(0);
	}
	else
	// a string or null
	//	return new Float(0);
	    throw new ParserException("#REFS?");

    }

    /**
     * Determines if a cell at these coordinates is a formula cell
     *
     * @param aRow row coordinate
     * @param aColumn column coordinate
     * @return true only if the cell at those coordinates is a formula
     */
    public boolean isFormula(int aRow, int aColumn) {
        UMCell temp = getCellAt(aRow, aColumn);
        return ((temp != null) && (temp.getType() == UMCell.FORMULA));
    }

    //************************************************************
    //Public Method: setCellAt
    //Description: this method assumes that the object passed to it
    //             is a correct object to set the value of the cell as
    //             so it sets the cell -- add support for the formulas later.
    //
    // Parameters:
    //      input      - data object for the cell
    //      aRow       - row coordinate
    //      aColumn    - column coordinate
    //*************************************************************
    public void setCellAt(Object input, int aRow, int aColumn) {
        UMCell temp = getCellAt(aRow, aColumn);

	/* if for some reason value out of bounds ignore */
        if (temp != null) {

	    //always remove references old formula referred to
	    removeRefs(aRow, aColumn);

	    //insert new formula
	    if (input instanceof Formula) {
		temp.setFormula((Formula)input);

		if (isLoop(new CellPoint(aRow, aColumn))) {
		    ParserException loop = new ParserException("#LOOP?");
                    Formula form2 = new Formula(input.toString(), aRow, aColumn, loop);
		    setCellAt(form2, aRow, aColumn);
		    getCellAt(aRow, aColumn).setValue(loop);
		    updateRefs(aRow, aColumn);
		    return;
                }else {

		    addRefs(aRow, aColumn);
                    Debug.println(" calling CHECKING THE FORMULAR ");
		    recalculate(aRow, aColumn);
		    updateRefs(aRow, aColumn);

                }
            }else {
                //treat as normal data cell
		temp.setData(input);
		updateRefs(aRow, aColumn);
            }
	}
    }

    /**
     * This method should be called with a cell is set as a formula
     * cell (although it does nothing if it is not a formula). When a formula
     * is entered, it may reference other cells. These cells need to be
     * notified that if they are changed, to notify this formula cell.
     * This method adds the cell coordinates to the reference list of each
     * cell that the formula references.
     *
     * @param aRow row of formula cell
     * @param aColumn column of formula cell
     */
    public void addRefs(int aRow, int aColumn) {
        if (isFormula(aRow, aColumn)) {
            Formula temp = getCellAt(aRow, aColumn).getFormula();
            TreeSet list = temp.getDependency();
//Debug.println("inside UMTablemodel of add refs"+ aRow +' '+ aColumn);
            /* use formula's dependency to find cells
             * that need to notify it if their values
             * change
             */
            Iterator it = list.iterator();
	    CellPoint thisRef = new CellPoint(aRow, aColumn);
	    while (it.hasNext()) {
                CellPoint update = (CellPoint)it.next();
		UMCell field = getCellAt(update.getRow(), update.getCol());

                //test of cell found was out of bounds
		if (field != null)
		    field.addRef(thisRef);
            }
        }
    }

    /**
     * This method removes this formula cell from the reference lists of all
     * cells it references. If this is not a formula cell, it does nothing.
     * This method should be called with a formula cell is being changed to a
     * non-Formula cell.
     *
     * @param aRow row of cell to remove from reference list
     * @param aColumn column of cell to remove from reference list
     */
    public void removeRefs(int aRow, int aColumn) {
        if (isFormula(aRow, aColumn)) {
            Formula temp = getCellAt(aRow, aColumn).getFormula();
            TreeSet list = temp.getDependency();

            /* use formula dependcy list to go to cells that it references
             * then remove its entry form their reference list
             */
            Iterator it = list.iterator();
	    CellPoint thisRef = new CellPoint(aRow, aColumn);
	    while (it.hasNext()) {
                CellPoint update = (CellPoint)it.next();
                UMCell field = getCellAt(update.getRow(), update.getCol());

		if (field != null)
		    field.removeRef(thisRef);

            }
        }
    }

    /**
     * This method updates the values of all cells that
     * reference this one. It recursively updates all cells that depend on this
     * one and cells that depend on those, etc.
     *
     * @param aRow row of cell to update
     * @param aColumn column of cell to update
     */
    public void updateRefs(int aRow, int aColumn) {

        UMCell temp = getCellAt(aRow, aColumn);
	if (temp == null)
	    return;

	TreeSet set = getRefs(aRow, aColumn);
	Debug.println("refs set for "+Node.translateColumn(aColumn)+
		      Node.translateRow(aRow)+": "+set);

	// mark it as "needsRecalc";
	Iterator it = set.iterator();
	while (it.hasNext()) {
	    CellPoint point = (CellPoint)it.next();
	    Formula formula = getCellAt(point.getRow(), point.getCol()).getFormula();

	    formula.setNeedsRecalc(true);
	    // make sure JTable refreshes it
	    //fireTableCellUpdated(point.getRow(), point.getCol());
	}

	// recalculate
	it = set.iterator();
	while (it.hasNext()) {
	    CellPoint point = (CellPoint)it.next();
	    try {
		getNumericValueAt(point.getRow(), point.getCol());
	    }
	    catch (ParserException e) {
	    }

	    // make sure JTable refreshes it
	    fireTableCellUpdated(point.getRow(), point.getCol());
	}

        //make sure to tell JTable things have changed
	fireTableCellUpdated(aRow, aColumn);
    }

    /**
     * This method gets the set of cells that will be affects
     * by a value change for the cpecified cell.
     *
     * @param row the row
     * @param col the column
     *
     */
    private TreeSet getRefs(int row, int col) {
	TreeSet set = new TreeSet();
	getRefs(row, col, set);
	return set;
    }

    /**
     * This method is a helper method for getReds(int, int).
     * It recursively gets refs for each cell and merges it into
     * the set.
     *
     * @param row the row
     * @param col the column
     * @param set the current of cells
     *
     */
    private void getRefs(int row, int col, TreeSet set) {
	UMCell cell = getCellAt(row, col);
	if (cell == null || !cell.hasRefs())
	    return;

	Iterator it = cell.getRefs().iterator();
	while (it.hasNext()) {
	    CellPoint point = (CellPoint)it.next();
	    set.add(point);
	    getRefs(point.getRow(), point.getCol(), set);
	}
    }
    //Billy added the function here
    /**
     * This method recalculates all cells in the table
     *
     * @see #insertRow
     * @see #insertColumn
     * @see #removeRow
     * @see #removeColumn
     *
     */
    public void recalculateAll() {
	for (int i = 1; i < getRowCount(); i++)
	    for (int j = 1; j < getColumnCount(); j++) {
		addRefs(i, j);
		recalculate(i, j);
	    }
    }
    /**
     * Starting from cell, detect potential reference loops.
     *
     * @param cell specified CellPoint
     */
    private boolean isLoop(CellPoint cell) {
	return isLoop(cell, new TreeSet());
    }

    /**
     * Starting from cell, detect potential reference loops.
     *
     * @param cell specified CellPoint
     */
    private boolean isLoop(CellPoint cell, TreeSet set) {
	if (set.contains(cell))
	    return true;

	UMCell objCell = getCellAt(cell.getRow(), cell.getCol());
	if (objCell == null)
	    return false;
	Formula formula = objCell.getFormula();
	if (formula == null)
	    return false;

	set.add(cell);

	Iterator it = formula.getDependency().iterator();
	while (it.hasNext()) {
	    CellPoint newCell = (CellPoint)it.next();
	    boolean ret = isLoop(newCell, set);
	    if (ret)
		return true;
	}

	set.remove(cell);
	return false;
    }

    //************************************************************
    //Private Static Method: fieldParser
    //Description: This is a static method that parses string input
    //             passed to it from somewhere else.  Later use this
    //             to determine the types of objects
    //
    // Parameters:
    //      input     - String to parse
    //      CellPoint  - the point in the table where this input
    //                   needs to be placed
    //*************************************************************
    private static Object fieldParser(String input, CellPoint c) {
        if (input == null)
	    return new String("");

        int row = c.getRow();
        int col = c.getCol();


	/* try making it a formula */
	if (input.startsWith("=")) {
	    Formula form = null;

            //create formula and its value and put in a cell
	    try {
	        return new Formula(input.substring(1), row, col);
            }catch (ParserException e) {
		// no parsing
		return new Formula(input.substring(1), row, col, e);
	    }

        }else {
	    /* try making it a number */
	    try {
	          return new Float(input);

            }catch (NumberFormatException e2) {
		/* all else fails treat as string */
		return input;
            }
        }
    }

    /** This method does not recognize formula strings. It is used for dialogue
     * box input where there will be no formulas inputted or expected to be
     * inputted.
     * @param input input string to parse
     * @return appropriate object after parsing
     */

    public static Object fieldParser(String input) {
        if (input == null)
	    return new String("");

	/* try making it a number */
	try {
	    return new Float(input);
        }catch (NumberFormatException e) {
	    /* all else fails treat as string */
	    return input;
        }
    }

    /**
     * This is a wraper method of the Formula class's evaluate method. This
     * method recalculates the value of a formula at the given coordinates.
     * If the coordinates do not specify a formula it does nothing.
     *
     * @param aRow the row coordinate
     * @param aColumn the column coordinate
     */

    public void recalculate(int aRow, int aColumn) {
        if (isFormula(aRow, aColumn)) {

	    try {
               // Debug.println("recalculate here billy");
		Number eVal = Formula.evaluate(this, aRow, aColumn);

		//we set the value here
                //Debug.println(aRow +"***"+ aColumn +"***");
		getCellAt(aRow, aColumn).setValue(eVal);
                //Debug.println("CHECKING THE FORMULAR "+ eVal);
	    }
	    catch (ParserException e) {
                //set value as the appropriate error message
		getCellAt(aRow, aColumn).setValue(e);
	    }
        }
    }
    /**
     * This is the version of recalculate that takes a CellPoint object as an
     * argument.
     *
     * @param x the coordinates of the cell to be updated
     */
    public void recalculate(CellPoint x) {
        recalculate(x.getRow(), x.getCol());
    }

     public String toString()
    {
      String text = toString(new CellRange(UMSpreadSheet.m_baseRow, getRowCount()-1,
        	                           UMSpreadSheet.m_baseCol, getColumnCount()-1), false);

      return text;
    }

    //********************************************************
    // toString is used to convert a range of cells into a string.
    // One row per line, and each column is tab-delimited.
    // @param range the range in the table
    // @param byValue get the value instead of a formula
    // @return a string
    // @see Formula#fixRelAddr
    // @see FileOp#saveTableModel
    // @see EditOp#cut
    // @see EditOp#copy
    // @see Clipboard
    //********************************************************
    public String toString(CellRange range, boolean byValue) {
	StringBuffer sbf=new StringBuffer();

	for (int i=range.getStartRow(); i<=range.getEndRow(); i++) {
	    for (int j=range.getStartCol(); j<=range.getEndCol(); j++) {
		if (byValue) {
		    sbf.append(getValueAt(i, j));
		}
		else {
		    UMCell cell = getCellAt(i, j);
		    // if cell is not empty
		    if(cell != null)
			sbf.append(cell.toString());
		}

		if (j<range.getEndCol())
		    sbf.append("\t");
	    }
	    sbf.append("\n");
	}
	String text = sbf.toString();
	return text;
    }

    //********************************************************
    // fromString is used to convert a string to valus in a range of cells.
    // One row per line, and each column is tab-delimited.
    // @param text the string
    // @param rowOff the row offset
    // @param colOff the column offset
    // @param range the range to paste
    // @see Formula#fixRelAddr
    // @see FileOp#openTableModel
    // @see EditOp#paste
    // @see Clipboard
    //********************************************************
    public void fromString(String text, int rowOff, int colOff, CellRange range) {

	try {
	    BufferedReader in = new BufferedReader(new StringReader(text));
	    String line;
	    int row = range.getStartRow();

	    while (row <= range.getEndRow()) {
		line = in.readLine();
                //System.out.println("------>>>>> " + line);

		int index;
		int prev = 0;

		// set col to startCol before each loop
		int col = range.getStartCol();
		String value;

		while (col <= range.getEndCol()) {
		    index = line.indexOf('\t', prev);
		    if (index >= 0) {
			value = line.substring(prev, index);
		    }
		    else {
			value = line.substring(prev);
		    }

		    if (value.startsWith("=")) {
				// need to fix relative address
			/*value =
			    Formula.fixRelAddr(value.substring(1),
					       rowOff, colOff);
			if (value == null)
			    value = new String("=$REFS$0");
			else
			    value = "="+value;*/
		    }

		    doSetValueAt(value, row, col);
		    //System.out.println(""+row+','+col+": "+value);

		    prev = index+1;
		    // increment column number
		    col++;

		    if (index == -1)
			break;
		}

		row++;
	    }
	}
	catch (Exception e) {
	}
    }
  //********************************************************
    // toString is used to convert a range of cells into a string.
    // One row per line, and each column is tab-delimited.
    // UM changes : appends code that tells font properties
    // @param range the range in the table
    // @param byValue get the value instead of a formula
    // @return a string
    // @see Formula#fixRelAddr
    // @see FileOp#saveTableModel
    // @see EditOp#cut
    // @see EditOp#copy
    // @see Clipboard
    //********************************************************
    public String toString1(CellRange range, boolean byValue) {
    //System.out.println("To String has been called");
	StringBuffer sbf=new StringBuffer();
        String cpy;
        int fontsize, indx, fontstate;
        int bGround = getTable().getBackground().getRGB(),
            fGround = getTable().getForeground().getRGB(),
            iTop = 0,
            iBot = 0,
            iLeft = 0,
            iRight = 0;
	for (int i=range.getStartRow(); i<=range.getEndRow(); i++) {
	    for (int j=range.getStartCol(); j<=range.getEndCol(); j++) {
		if (byValue) {
		    sbf.append(getValueAt(i, j));
		}
		else {
		    UMCell cell = getCellAt(i, j);
		    // if cell is not empty
		    if(cell != null){
			sbf.append(cell.toString());
                      try{
                        if(cell.getFont()== null){
                         //System.out.print("Cell Get font is null Initialising Inddex");
                         indx=0;
                         fontsize=12;
                         fontstate=0;
                        }
                        else{
                          indx=EditMenuOp.getFontIndex(cell.getFont().getName());
                          fontsize=cell.getFont().getSize();
                          if(cell.getFont().isPlain())
                            fontstate=0;
                          else if(cell.getFont().isItalic())
                            fontstate=1;
                          else
                            fontstate=2;
                        }
                        Color bColor = cell.getBackground();
                        Color fColor = cell.getForeground();
                        LinesBorder border = (LinesBorder) cell.getBorder();
                        bGround = getTable().getBackground().getRGB();
                        fGround = getTable().getForeground().getRGB();
                        iTop = 0;
                        iBot = 0;
                        iLeft = 0;
                        iRight = 0;

                        if (bColor != null)
                            bGround = bColor.getRGB();
                        if (fColor != null)
                            fGround = fColor.getRGB();
                        if (border != null)
                        {
                            //System.out.println("Border is not null");
                            Insets ins = border.getBorderInsets(null);
                            iTop = ins.top;
                            iBot = ins.bottom;
                            iRight = ins.right;
                            iLeft = ins.left;
                        }

                        sbf.append("\\*"+indx+"*"+fontsize+"*"+fontstate+"*"+
                                   bGround+"*" + fGround + "*" + iTop +
                                   "*" + iBot + "*" + iRight + "*" +
                                   iLeft + "*/");

                        } catch (Exception e){
                          System.out.print("Error Occurance");
                          e.printStackTrace();
                        }
                    }
		}

		if (j<range.getEndCol()){
		    sbf.append("\t");
                  }
	    }
	    sbf.append("\n");
	}
	String text = sbf.toString();

	return text;
    }
    //********************************************************
    // fromString is used to convert a string to valus in a range of cells.
    // One row per line, and each column is tab-delimited.
    // @param text the string
    // @param rowOff the row offset
    // @param colOff the column offset
    // @param range the range to paste
    // @see Formula#fixRelAddr
    // @see FileOp#openTableModel
    // @see EditOp#paste
    // @see Clipboard
    //********************************************************

    public void fromString1(String text,
                            int rowOff,
                            int colOff,
                            CellRange range)
    {
        boolean camein=false;
	try {
	    BufferedReader in = new BufferedReader(new StringReader(text));
	    String line;
	    int row = range.getStartRow();

	    while (row <= range.getEndRow()) {
		line = in.readLine();
                //System.out.println("------>>>>> \n" + line);

		int index;
		int prev = 0, prev2=0;

		// set col to startCol before each loop
		int col = range.getStartCol(),loc;
		String value, temp, start1, fontproperty=null;
                start1= "\\*";
		while (col <= range.getEndCol()) {
                    camein=false;
		    index = line.indexOf('\t', prev);
		    if (index >= 0) {
			//value= line.substring(prev, index);
                        temp = line.substring(prev,index);
                        //System.out.println("Temp: "+temp);
                        //WILL ALWAYS FIND START1
                        loc=temp.indexOf(start1, prev2);


                        if(loc>=0){
                          value=temp.substring(0,loc);
                          fontproperty=temp.substring(loc);
                        }
                        else
                          value=temp;
		    }
		    else {
                        temp = line.substring(prev);

                        //System.out.println("Temp: "+temp+" Code 3 Index"+index);
                        //WILL ALWAYS FIND START1
                        loc=temp.indexOf(start1, prev2);
                        if(loc>=0){
                          value=temp.substring(0,loc);
                          fontproperty=temp.substring(loc);
                        }
                        else
                          value=temp;
		    }

		    if (value.startsWith("=")) {
				// need to fix relative address
                        //System.out.println("Temp: "+temp);
                        String ret=Formula.fixRelAddr(value.substring(1),
					       rowOff, colOff);

			if (value == null)
			    value = new String("=$REFS$0");
			else
			    value = "="+ret;

                        Formula form = null;

		        //create formula and its value and put in a cell

		        try {
		          form = new Formula(value.substring(1),row, col);
                          setCellAt(form, row,col);
                          camein=true;
                          //set font properties
                          setFontproperty(fontproperty, getCellAt(row, col));
		          }
		        catch (ParserException e) {
		        // no parsing
		            form = new Formula(value.substring(1), row,col, e);
                            setCellAt(form, row,col);
                            camein=true;
                            getCellAt(row,col).setValue(e);
                            setFontproperty(fontproperty, getCellAt(row, col));
                        }
		    }
		    if(!camein)
                      setCellAt(value,row, col,fontproperty);
		    prev = index+1;
		    // increment column number
		    col++;
		    if (index == -1)
			break;
		}
		row++;
	    }
	}
	catch (Exception e) {
	}
    }

    //similar copy of SetCellAt
    public void setCellAt(Object input, int aRow, int aColumn, String fontdata) {
        //System.out.println("Set cell At UMCell Called");
        UMCell temp = getCellAt(aRow, aColumn);

	/* if for some reason value out of bounds ignore */
        if (temp != null) {
	    //always remove references old formula referred to
	    removeRefs(aRow, aColumn);
	    //insert new formula
	    if (input  instanceof Formula) {
		temp.setFormula((Formula) input);

		if (isLoop(new CellPoint(aRow, aColumn))) {
		    ParserException loop = new ParserException("#LOOP?");
                    Formula form2 = new Formula(input.toString(), aRow, aColumn, loop);
		    setCellAt(form2, aRow, aColumn);
		    getCellAt(aRow, aColumn).setValue(loop);
		    updateRefs(aRow, aColumn);
		    return;
                }else {

		    addRefs(aRow, aColumn);
		    recalculate(aRow, aColumn);
		    updateRefs(aRow, aColumn);
                }
            }else
            {
            /* Default font values
                if (fontdata != null)
                {
                int fname=0, size=12, state=0,
                    bGround = getTable().getBackground().getRGB(),
                    fGround = getTable().getForeground().getRGB(),
                    iTop= 0,
                    iBot = 0,
                    iRight = 0,
                    iLeft = 0;
                String indx,fontsize, fontstate,
                    bColorStr, fColorStr,
                    iTopStr, iBotStr, iLeftStr,
                    iRightStr;
                String parseStr = fontdata.substring(2,fontdata.length());
                //System.out.println(parseStr);
                if(fontdata != null){
                  StringTokenizer tk = new StringTokenizer(parseStr, "*");
                  try {
                  indx= tk.nextToken();
                  fontsize= tk.nextToken();
                  fontstate = tk.nextToken();
                  bColorStr = tk.nextToken();
                  fColorStr = tk.nextToken();
                  iTopStr = tk.nextToken();
                  iBotStr = tk.nextToken();
                  iRightStr = tk.nextToken();
                  iLeftStr = tk.nextToken();


                    fname=Integer.parseInt(indx);
                    size=Integer.parseInt(fontsize);
                    state=Integer.parseInt(fontstate);
                    bGround = Integer.parseInt(bColorStr);
                    fGround = Integer.parseInt(fColorStr);
                    iTop = Integer.parseInt(iTopStr);
                    iBot = Integer.parseInt(iBotStr);
                    iRight = Integer.parseInt(iRightStr);
                    iLeft = Integer.parseInt(iLeftStr);
                    } catch (Exception e){
                      System.out.println("Format Error in SetCellAt");
                      input = new String(input.toString()) + fontdata;
                  }
                }

                Font a;
                if(state==0)
                  a=new Font(Fontlist[fname], Font.PLAIN,size);
                else if(state==1)
                  a=new Font(Fontlist[fname], Font.ITALIC,size);
                else
                  a=new Font(Fontlist[fname], Font.BOLD,size);
                temp.setFont(a);

                Color bColor = new Color(bGround);
                Color fColor = new Color(fGround);
                LinesBorder border = null;
                if (iTop == 2 ||
                    iBot == 2 ||
                    iRight == 2 ||
                    iLeft == 2)
               {
                   Insets ins = new Insets(1,1,1,1);
                   ins.top = iTop;
                   ins.bottom = iBot;
                   ins.left = iLeft;
                   ins.right = iRight;
                   border = new LinesBorder(Color.black, ins);
               }
                temp.setBackground(bColor);
                temp.setForeground(fColor);
                temp.setBorder(border);
                }*/

                //FONT ACTIVITIES
                try{
                  setFontproperty(fontdata, temp);
                } catch( Exception e){
                  System.out.println("Format Error in SetCellAt");
                  input = new String(input.toString()) + fontdata;
                }
		//temp.copy(input);
                if (input == null)
	            input = new String("");
                if( input instanceof String){
                try{
                    String hope=(String) input;
                    Float data = new Float(hope);
                    temp.setData(data);
	        }
                catch (NumberFormatException e){
                    //string and formulas
                      temp.setData(input);
                }
                }

		updateRefs(aRow, aColumn);

            }

	}
    }
    private void setFontproperty(String fontdata, UMCell temp) throws Exception{
      if (fontdata != null){
        int fname=0, size=12, state=0, bGround = getTable().getBackground().getRGB(),
          fGround = getTable().getForeground().getRGB(),
          iTop= 0,
          iBot = 0,
          iRight = 0,
          iLeft = 0;
        String indx,fontsize, fontstate,bColorStr, fColorStr,iTopStr, iBotStr, iLeftStr,iRightStr;
        String parseStr = fontdata.substring(2,fontdata.length());
                //System.out.println(parseStr);
        if(fontdata != null){
          StringTokenizer tk = new StringTokenizer(parseStr, "*");
          try {
            indx= tk.nextToken();
            fontsize= tk.nextToken();
            fontstate = tk.nextToken();
            bColorStr = tk.nextToken();
            fColorStr = tk.nextToken();
            iTopStr = tk.nextToken();
            iBotStr = tk.nextToken();
            iRightStr = tk.nextToken();
            iLeftStr = tk.nextToken();

            fname=Integer.parseInt(indx);
            size=Integer.parseInt(fontsize);
            state=Integer.parseInt(fontstate);
            bGround = Integer.parseInt(bColorStr);
            fGround = Integer.parseInt(fColorStr);
            iTop = Integer.parseInt(iTopStr);
            iBot = Integer.parseInt(iBotStr);
            iRight = Integer.parseInt(iRightStr);
            iLeft = Integer.parseInt(iLeftStr);
          }catch (Exception e){
            throw new Exception("Font Property Error");
          }
        }

        Font a;
          if(state==0)
            a=new Font(Fontlist[fname], Font.PLAIN,size);
          else if(state==1)
            a=new Font(Fontlist[fname], Font.ITALIC,size);
          else
            a=new Font(Fontlist[fname], Font.BOLD,size);
          temp.setFont(a);

          Color bColor = new Color(bGround);
          Color fColor = new Color(fGround);
          LinesBorder border = null;
          if (iTop == 2 ||iBot == 2 ||iRight == 2 ||iLeft == 2){
            Insets ins = new Insets(1,1,1,1);
            ins.top = iTop;
            ins.bottom = iBot;
            ins.left = iLeft;
            ins.right = iRight;
            border = new LinesBorder(Color.black, ins);
          }
          temp.setBackground(bColor);
          temp.setForeground(fColor);
          temp.setBorder(border);
      }

}
    public String ConvertToString()
    {
      String text = CelltoString(new CellRange(UMSpreadSheet.m_baseRow,
                                               getRowCount()-1,
                                               UMSpreadSheet.m_baseCol,
                                               getColumnCount()-1), false);


      return text;
    }
    //********************************************************
    // toString is used to convert a range of cells into a string.
    // One row per line, and each column is tab-delimited.
    // @param range the range in the table
    // @param byValue get the value instead of a formula
    // @return a string
    // @see Formula#fixRelAddr
    // @see FileOp#saveTableModel
    // @see EditOp#cut
    // @see EditOp#copy
    // @see Clipboard
    //********************************************************
    public String CelltoString(CellRange range, boolean byValue) {
	StringBuffer sbf=new StringBuffer();
        //System.out.println("IN CELL TO STRING");
	for (int i=range.getStartRow(); i<=range.getEndRow(); i++) {
	    for (int j=range.getStartCol(); j<=range.getEndCol(); j++) {
		if (byValue) {
		    sbf.append(getValueAt(i, j));
		}
		else {
		    UMCell cell = getCellAt(i, j);
		    // if cell is not empty
		    if(cell != null)
			sbf.append(cell.CellToString());
		}

		if (j<range.getEndCol())
		    sbf.append("\t");
	    }
	    sbf.append("\n");
	}
	String text = sbf.toString();
        //System.out.println(text);
        //System.out.println("======================================");
	return text;
    }

        //********************************************************
    // fromString is used to convert a string to valus in a range of cells.
    // One row per line, and each column is tab-delimited.
    // @param text the string
    // @param rowOff the row offset
    // @param colOff the column offset
    // @param range the range to paste
    // @see Formula#fixRelAddr
    // @see FileOp#openTableModel
    // @see EditOp#paste
    // @see Clipboard
    //********************************************************
    public void CellfromString(String text, int rowOff, int colOff, CellRange range) {

	try {
	    BufferedReader in = new BufferedReader(new StringReader(text));
	    String line;
	    int row = range.getStartRow();

	    while (row <= range.getEndRow()) {
		line = in.readLine();
                //System.out.println("------>>>>> " + line);

		int index;
		int prev = 0;

		// set col to startCol before each loop
		int col = range.getStartCol();
		String value;

		while (col <= range.getEndCol()) {
		    index = line.indexOf('\t', prev);
		    if (index >= 0) {
			value = line.substring(prev, index);
		    }
		    else {
			value = line.substring(prev);
		    }

                    //System.out.println(""+row+','+col+": "+value);
		    doSetCellValueAt(value, row, col);
		    //System.out.println(""+row+','+col+": "+value);

		    prev = index+1;
		    // increment column number
		    col++;

		    if (index == -1)
			break;
		}

		row++;
	    }
       	    for (int i=range.getStartRow(); i<=range.getEndRow(); i++) {
	      for (int j=range.getStartCol(); j<=range.getEndCol(); j++) {
                updateRefs(i, j);
              }
            }
	}
	catch (Exception e) {
	}
    }

    public void doSetCellValueAt(String aValue, int aRow, int aColumn)
    {
      //System.out.println("############ "+aValue);
      m_modified = true;
      UMCell newCell = new UMCell();
      String temp = new String();
      StringTokenizer strToken = new StringTokenizer((String)aValue, "/");
      Color bColor = null;
      Color fColor = null;
      Font font = null;
      Object value = null;
      Formula expression = null;
      UMCell tempCell = getCellAt(aRow, aColumn);

      //parse out the background color
      temp = strToken.nextToken();
      //System.out.println("Back Color ===> " + temp);
      if (temp.compareTo("null") != 0)
      {
        StringTokenizer intToken = new StringTokenizer(temp, "-");
        int blue = (Integer.valueOf(intToken.nextToken())).intValue();
        int red = (Integer.valueOf(intToken.nextToken())).intValue();
        int green = (Integer.valueOf(intToken.nextToken())).intValue();
        //System.out.println(blue + " = " + red + " = " + green + " = ");
        bColor = new Color (red, green, blue);
        //newCell.setBackground(bColor);
        tempCell.setBackground(bColor);
        //System.out.println("======= >>> " + newCell.getBackground());
      }

      //parse out the foreground color
      temp = strToken.nextToken();
      //System.out.println("Fore Color ===> " + temp);
      if (temp .compareTo("null") != 0)
      {
        StringTokenizer intToken = new StringTokenizer(temp, "-");
        int blue = (Integer.valueOf(intToken.nextToken())).intValue();
        int red = (Integer.valueOf(intToken.nextToken())).intValue();
        int green = (Integer.valueOf(intToken.nextToken())).intValue();
        //System.out.println(blue + " = " + red + " = " + green + " = ");
        fColor = new Color (red, green, blue);
        //newCell.setForeground(fColor);
        tempCell.setForeground(fColor);
      }

      //parse out the font style
      temp = strToken.nextToken();
      //System.out.println("Font ===> "+temp);
      if (temp.compareTo("null") != 0)
      {
        StringTokenizer fontToken = new StringTokenizer(temp, "-");
        String name = fontToken.nextToken();
        int size = (Integer.valueOf(fontToken.nextToken())).intValue();
        int style = (Integer.valueOf(fontToken.nextToken())).intValue();
        //System.out.println(name + " = " + size + " = " + style + " = ");
        font = new Font(name, style, size);
        //newCell.setFont(font);
        tempCell.setFont(font);
        //font = Font.getFont(temp);
        //System.out.println("Font ===> "+font);
      }

      temp = strToken.nextToken();
      if (temp.compareTo("null") != 0)
      {
        StringTokenizer borderToken = new StringTokenizer(temp, "-");
        int top = (Integer.valueOf(borderToken.nextToken())).intValue();
        int bottom = (Integer.valueOf(borderToken.nextToken())).intValue();
        int left = (Integer.valueOf(borderToken.nextToken())).intValue();
        int right = (Integer.valueOf(borderToken.nextToken())).intValue();
        Insets ins = new Insets(top, left, bottom, right);
        Border theBorder = new LinesBorder
            (UIManager.getColor("Table.gridColor"), ins);
        //newCell.setBorder(theBorder);
        tempCell.setBorder(theBorder);
      }
      //setValueAt(newCell, aRow, aColumn);
      if (strToken.hasMoreElements())
      {
        temp = strToken.nextToken();
        doSetValueAt(temp, aRow, aColumn);
      }
    }




}
